package com.acj0.orangediaryproa.mod.backup;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Scanner;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserFactory;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Build;
import android.text.format.DateUtils;
import android.util.Log;

import com.acj0.orangediaryproa.MyPref;
import com.acj0.orangediaryproa.PrefBackupDrbx;
import com.acj0.orangediaryproa.PrefBackupLocal;
import com.acj0.orangediaryproa.data.DBAdapter;
import com.acj0.orangediaryproa.data.EntryOper;
import com.acj0.orangediaryproa.data.MyApp;
import com.acj0.share.ShareApp;
import com.acj0.share.mod.backup.Exporter;
import com.acj0.share.utils.IOUtils;
import com.acj0.share.utils.LogUtils;
import com.acj0.share.utils.Util;
import com.acj0.share.utils.XmlUtils;

public class Backup {

	private static final String TAG = "Backup";
	
	public static final int BACKUP_MODE_ACTIVITY = 0;	
	public static final int BACKUP_MODE_SERVICE = 1;	
	
	public static final SimpleDateFormat SDF_BACKUP = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
	public static final String TEMP_FILE_NAME = "data.tmp";
	public static final String BACKUP_LOG = MyApp.BASE_DIR + File.separator + LogUtils.FILE_NAME[LogUtils.LOG_BACKUP];
	public static final String ACTION_BACKUP = MyApp.CURR_VERSION_PACKAGE + ".ACTION_BACKUP";

	public static final String TRANSFER_DIR_LITE =	MyApp.ROOT_DIR + File.separator + MyApp.DEFAULT_DIR_LITE + File.separator + "backup";
	public static final String TRANSFER_DIR_DEMO =	MyApp.ROOT_DIR + File.separator + MyApp.DEFAULT_DIR_DEMO + File.separator + "backup";
	public static final String TRANSFER_DIR_ADVR =	MyApp.ROOT_DIR + File.separator + MyApp.DEFAULT_DIR_ADVR + File.separator + "backup";
	
	public Context mContext;
	private DBAdapter mDbHelper;
	
	private int mBackupMode;
	
	public int mErrorCode = 0;
	public String mErrorMsg = "";
	
	
	// Constructor
	public Backup(Context context, int backupMode) {
		mContext = context;
		mDbHelper = new DBAdapter(context);
		mBackupMode = backupMode;
	}
	
	public static long getIntervalTime(int key) {
		switch (key) {
		case 0: return DateUtils.HOUR_IN_MILLIS * 12;
		case 1: return DateUtils.DAY_IN_MILLIS * 1;
		case 2: return DateUtils.DAY_IN_MILLIS * 3;
		case 3: return DateUtils.DAY_IN_MILLIS * 7;
		}
		return 0;
	}
	
	
	// db image backup error: 100s
	public void backupDat1(String target) {
		
		mDbHelper.open();
		
		String currentDbFile = mDbHelper.getDBPath();
		
		mErrorCode = 999;
		mErrorMsg = "";

		// Copy database image to target backup file

		if (ShareApp.DEBUG) Log.e(TAG, "Restore 1. Check source file status");
//		String result1 = IOUtils.cp(mDatabaseFile, target);
		boolean result1 = false;
		try {
			result1 = mDbHelper.exportDatabase1(currentDbFile, target);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		if (result1) {
			mErrorCode = 0;
			mErrorMsg = "Created database backup";
		}
		else {
			mErrorCode = 100;
			mErrorMsg = "Cannot create backup file\n" + result1;
		}
		if (ShareApp.DEBUG) Log.e(TAG, "backupDat1: "+currentDbFile+" ... "+target);
		if (ShareApp.DEBUG) Log.e(TAG, "backupDat1: "+mErrorMsg);
		
		mDbHelper.close();
	}
	
	
	public void restoreDat1(String source) {

		mDbHelper.open();
		String currentDbFile = mDbHelper.getDBPath();
		
		mErrorCode = 999;
		mErrorMsg = "";

		String temp = MyApp.BACKUP_DIR + File.separator + TEMP_FILE_NAME;

		if (ShareApp.DEBUG)	Log.e(TAG, "Restore 1. Check source file status");

		File sourcef = new File(source);
		if (sourcef == null || !sourcef.isFile()) {
			mErrorCode = 200;
			mErrorMsg = "Source file not found. File:" + source;
			if (ShareApp.DEBUG)	Log.e(TAG, mErrorMsg);
			return;
		} 
		else {
			if (ShareApp.DEBUG)	Log.e(TAG, "Source file found. File:" + source);
		}

		try {
			Scanner sc = new Scanner(sourcef);
			String s1 = sc.next();
			String s2 = sc.next();
			if (s1.equals("SQLite") && s2.equals("format")) {
				if (ShareApp.DEBUG)	Log.e(TAG, "Good! signature found:" + s1 + " " + s2);
			} 
			else {
				mErrorCode = 201;
				mErrorMsg = "Source is not right backup file. No signature found!";
				if (ShareApp.DEBUG)	Log.e(TAG, mErrorMsg);
				return;
			}
		} 
		catch (Exception e) {
			mErrorCode = 202;
			mErrorMsg = "Exception occured while reading file " + e.getMessage();
			return;
		}

		if (ShareApp.DEBUG)	Log.e(TAG, "Restore 2. Create backup of current db image");
		String result = IOUtils.cp(currentDbFile, temp);
		if (result.substring(0, 7).equals("Failed!")) {
			mErrorCode = 203;
			mErrorMsg = "Cannot create current db image backup\n" + result;
			if (ShareApp.DEBUG)	Log.e(TAG, mErrorMsg);
			return;
		} 
		else {
			if (ShareApp.DEBUG)	Log.e(TAG, "Good! Temp file created: " + temp);
		}

		// Overwrite database with source
		if (ShareApp.DEBUG)	Log.e(TAG, "Restore 3. Update db image");
		boolean result1 = false;
		
		try {
			result1 = mDbHelper.importDatabase1(source, currentDbFile);
			
			if (result1) {
				mErrorCode = 0;
				mErrorMsg = "Database restored";
				if (ShareApp.DEBUG) Log.e(TAG, "Good! db image overwritten");
			}
			else {
				mErrorCode = 204;
				mErrorMsg = "Cannot update current db image with backup\n" + result1;
				if (ShareApp.DEBUG) Log.e(TAG, mErrorMsg);
			}
		}
		catch (Exception e) {
			mErrorCode = 205;
			mErrorMsg = "Cannot update current db image with backup\n" + e.getMessage();
			if (ShareApp.DEBUG) Log.e(TAG, mErrorMsg);
			e.printStackTrace();
		}
		
		mDbHelper.close();
	}
	
	// xml backup. error: 300s
	public String createSysInfoFile(File f) {
		
		String s0 = Util.getDeviceInfoBackup(mContext);
		
		IOUtils.stringIntoFile(f, s0, false, false);
		
		if (f.isFile())		return f.getAbsolutePath();
		else				return null;
		
	}
	
	public boolean IsBackupDatMadeInSameSys(File f) {
		
		if (f == null || !f.isFile() || f.length()==0) {
			if (ShareApp.DEBUG)	Log.e(TAG, "IsBackupDatMadeInSameSys: SysInfoFile not exist");
			return false;
		} 
		
		try {
			Scanner scan = new Scanner(f);
			String line = scan.nextLine();
//			while (scan.hasNextLine()) {
//				String line = scan.nextLine();
//				System.out.println(line);
//			}

			if (ShareApp.DEBUG)	Log.e(TAG, "IsBackupDatMadeInSameSys: line: "+line);
			if (line.equals("Model: "+Build.MODEL))		return true;
			else										return false;
		} 
		catch (Exception e) {
			if (ShareApp.DEBUG)	Log.e(TAG, "IsBackupDatMadeInSameSys: Exception occured while reading file " + e.getMessage());
			return false;
		}
		
		
	}
	
	// xml backup. error: 300s
	public void backupXml(String targetFile) {

		mDbHelper.open();
		
		SQLiteDatabase db = mDbHelper.mDb;
		
		Exporter exporter;
		String[] tables = DBAdapter.TABLES_IN_USE;
		
		mErrorCode = 999;
		mErrorMsg = "";

		int tablesCount = tables.length;
		int recordCount = 0;

		// Check target status
		File targetf = new File(targetFile);
		// Log.e(TAG,"Check target file status: Exist:"+targetf.isFile()+", Readable:"+targetf.canRead()+", Writable:"+targetf.canWrite());
		// if (targetf==null || !targetf.isFile() || !targetf.canWrite()) {
		// mErrorCode = 300;
		// mErrorMsg = "Target file is either null, not file, or not writable";
		// return;
		// }
		//		
		try {
			FileOutputStream fos = new FileOutputStream(targetf);
			BufferedOutputStream bos = new BufferedOutputStream(fos);

			exporter = new Exporter(bos);
			exporter.writeDeclaration();
			exporter.startDbExport(db.getPath());

			// processing tables
			String tableName;
			for (int i = 0; i < tablesCount; i++) {
				tableName = tables[i];
				exporter.startTable(tableName);

				// get everything from the table
				String sql1 = "select * from " + tableName;
				Cursor c1 = db.rawQuery(sql1, new String[0]);
				int numrows = c1.getCount();

				if (ShareApp.DEBUG) Log.e(TAG, "Table " + tableName + " row count " + numrows);
				for (int j = 0; j < numrows; j++) {
					c1.moveToPosition(j);
					int numcols = c1.getColumnCount();
					exporter.startRow();
					String name;
					String val;
					for (int k = 0; k < numcols; k++) {
						name = c1.getColumnName(k);
						val = c1.getString(k);
						if (val == null) val = "";
						else
							val = XmlUtils.checkForIllegalChars(val);
						// if (Global.DEBUG) Log.e(TAG, "col '" + name +
						// "' -- val '");
						exporter.addColumn(name, val);
					}
					exporter.endRow();
					
					if (mBackupMode==BACKUP_MODE_ACTIVITY) {
						recordCount++;
						((PrefBackupLocal) mContext).setProgressMessage("Creating backup\nRecord exported: "+recordCount);
					}
				}

				c1.close();

				exporter.endTable();
				// if (Global.DEBUG) Log.e(TAG, "Table " + tableName +
				// " export end");
			}
			exporter.endDbExport();
			exporter.close();
			bos.close();
			fos.close();

			mErrorCode = 0;
			mErrorMsg = "Success";


		}
		catch (Exception e) {
			e.printStackTrace();
			mErrorCode = 301;
			mErrorMsg = "Failed!\nbackupXML exception occured" + e.toString();
			if (ShareApp.DEBUG) Log.e(TAG, "backupXML exception occured" + e.toString());
		}

		mDbHelper.close();

	}
	
	
    // ===================================
    // package dependant modules
    // ===================================
	
	
	public static final String[][] DROPBOX_BACKUP_DIR= {
		{	MyApp.PHOTO_DIR+File.separator,		"/photo/"		},
		{	MyApp.VOICE_DIR+File.separator,		"/voice/"		},
//		{	MyApp.EXPORT_DIR+File.separator,	"/export/"		},
//		{	MyApp.AUTOEXP_DIR+File.separator,	"/autoexp/"		},
    }; 
	
	public static final String[][] DROPBOX_RESTORE_DIR= {
		{	MyApp.PHOTO_DIR+File.separator,		"/photo/"		},
		{	MyApp.VOICE_DIR+File.separator,		"/voice/"		},
//		{	MyApp.AUTOEXP_DIR+File.separator,	"/autoexp/"		},
    }; 
		
	private class EntrData {		// total field count: 8
		public long _id;
		public String title;
		public String body;
		public long created;
		public int groupid;
		public String trash;
		public long remindertime;
		public int remindertype;
		public long folderid;
		
		public EntrData() {
			_id=0L; 
			title=""; 
			body=""; 
			created=System.currentTimeMillis();
			groupid=MyPref.VAL_MARKER_DEFAULT_ID;
			trash=EntryOper.TYPE_NORMAL; 
			remindertime=0L; 
			remindertype=0; 
			folderid=0;
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.FLD_ENTR_ID)) 					{	try { _id=Long.parseLong(val); 			} catch (Exception e) { } } 
			else if (key.equals(DBAdapter.FLD_ENTR_TITLE)) 			{	title=val; }
			else if (key.equals(DBAdapter.FLD_ENTR_BODY)) 			{	body=val; } 
			else if (key.equals(DBAdapter.FLD_ENTR_CREATED)) 		{	try { created=Long.parseLong(val); 		} catch (Exception e) { } }
			else if (key.equals(DBAdapter.FLD_ENTR_GROUPID)) 		{	try { groupid=Integer.parseInt(val); 	} catch (Exception e) { } }
			else if (key.equals(DBAdapter.FLD_ENTR_TRASH)) 			{	trash=val; }
//			else if (key.equals(DBAdapter.KEY_NOTE_REMINDER_TIME)) 	{	remindertime=val; }
//			else if (key.equals(DBAdapter.KEY_NOTE_REMINDER_TYPE)) 	{	remindertype=val; }
			else if (key.equals(DBAdapter.FLD_ENTR_FLD_ID)) 		{	try { folderid=Long.parseLong(val); 		} catch (Exception e) { } }
		}
		
		public long createRecord(DBAdapter dbHelper) {
			long entryid = -1;
			entryid = dbHelper.createNoteForRestore(_id, title, body, created, groupid, trash, remindertime, remindertype, folderid);
			if (entryid>0) {
				new EntryOper(mContext, dbHelper).createAutoExportXML(entryid, title, entryid, groupid, body);
			}
			return entryid;
		}
	}
	
	private class ItemData {		// total field count: 7
		public long _id;
		public long objid;
		public String title;
		public String fileUri;
		public String contentUri;
		public int type;
		public int int0;
		
		public ItemData() {
			_id=0L; 
			objid=0L; 
			title=""; 
			fileUri="";
			contentUri="";
			type=0;
			int0=0;
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.FLD_ITEM_ID)) 					{	try { _id=Long.parseLong(val); 			} catch (Exception e) { } } 
			else if (key.equals(DBAdapter.FLD_ITEM_NOTEID)) 		{	try { objid=Long.parseLong(val); 		} catch (Exception e) { } } 
			else if (key.equals(DBAdapter.FLD_ITEM_TITLE)) 			{	title=val; } 
			else if (key.equals(DBAdapter.FLD_ITEM_FILEURI)) 		{	fileUri=val; } 
			else if (key.equals(DBAdapter.FLD_ITEM_TYPE)) 			{	try { type=Integer.parseInt(val); 		} catch (Exception e) { } } 
			else if (key.equals(DBAdapter.FLD_ITEM_INT0)) 			{	try { int0=Integer.parseInt(val); 		} catch (Exception e) { } } 
		}
		
		public long createRecord(DBAdapter dbHelper) {
			return dbHelper.createItemForRestore(_id, objid, title, fileUri, type, int0);
		}
	}

	private class LblmData {			// total field count: 2
		public String label;
		public int markerid;
		
		public LblmData() {
			label=""; 
			markerid = -1;
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.FLD_LBLM_LABEL)) 			{	label=val;  } 
			else if (key.equals(DBAdapter.FLD_LBLM_MARKERID)) 	{	try { markerid=Integer.parseInt(val); 		} catch (Exception e) { } }
		}
		
		public long createRecord(DBAdapter dbHelper) {
			return dbHelper.createLabelMForRestore(label, markerid);
		}
	}
	
	private class Lb2mData {			// total field count: 2
		public long id;
		public int type;
		public String title;
		public int markerid;
		public long parent;
		
		public Lb2mData() {
			id=0L; 
			type=0; 
			title=""; 
			markerid=-1;
			parent = 0;
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.FLD_LB2M_ID)) 				{	try { id=Long.parseLong(val); 			} catch (Exception e) { } }
			else if (key.equals(DBAdapter.FLD_LB2M_TYPE)) 		{	try { type=Integer.parseInt(val); 		} catch (Exception e) { } }
			else if (key.equals(DBAdapter.FLD_LB2M_TITLE)) 		{	title=val;  } 
			else if (key.equals(DBAdapter.FLD_LBLM_MARKERID)) 	{	try { markerid=Integer.parseInt(val); 	} catch (Exception e) { } }
			else if (key.equals(DBAdapter.FLD_LB2M_PARENT))		{	try { parent=Long.parseLong(val); 		} catch (Exception e) { } }
		}
		
		public long createRecord(DBAdapter dbHelper) {
			return dbHelper.createLabel2MForRestore(id, type, title, markerid, parent);
		}
	}

	private class LbliData {			// total field count: 2
		public long noteid;
		public String label;
		
		public LbliData() {
			noteid=0L; 
			label=""; 
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.KEY_LBLI_ENTR_ID)) 			{	try { noteid=Long.parseLong(val); 	} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_LBLI_LBLM_ID)) 	{	label=val;  } 
		}
		
		public long createRecord(DBAdapter dbHelper) {
			return dbHelper.createLabelIForRestore(noteid, label);
		}
	}
	
	private class TaskData {			// total field count: 9
		public long id;
		public String title;
		public int completed;
		public int seq;
		public long listId;
		public long due;
		public int recType;
		
		public TaskData() {
			id=0L; 
			title=""; 
			completed=0; 
			seq=0; 
			listId=0L; 
			due=0L; 
			recType=0; 
		}
		
		public void setData(String key, String val) {
			if (key.equals(DBAdapter.FLD_TASK_ID)) 					{	try { id=Long.parseLong(val); 				} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_TASK_TITLE)) 		{	title=val;  } 
//			 else if (key.equals(DBAdapter.FLD_TODO_REMINDERTYPE)) 	{	reminderType=val;  } 
			 else if (key.equals(DBAdapter.FLD_TASK_COMPLETED)) 	{	try { completed=Integer.parseInt(val); 		} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_TASK_SEQ)) 			{	try { seq=Integer.parseInt(val); 			} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_TASK_LISTID)) 		{	try { listId=Long.parseLong(val); 			} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_TASK_DUE)) 			{	try { due=Long.parseLong(val); 				} catch (Exception e) { } } 
//			 else if (key.equals(DBAdapter.FLD_TODO_REMINDERTIME)) 	{	try { reminderTime=Long.parseLong(val); 	} catch (Exception e) { } } 
			 else if (key.equals(DBAdapter.FLD_TASK_RECTYPE)) 		{	try { recType=Integer.parseInt(val); 		} catch (Exception e) { } } 
		}
		
		public long createRecord(DBAdapter dbHelper) {
			return dbHelper.createTaskForRestore(id, title, completed, seq, listId, due, recType);
		}
	}


	// xml restore error: 400s
	public void restoreXml(String source) {
		
		mDbHelper.open();
		
		int insertCount = 0;
		
		mErrorCode = 999;
		mErrorMsg = "";
		
		if (ShareApp.DEBUG) Log.e(TAG, "Restore 1. Check source file status");
		
		File sourcef = new File(source);
		if (sourcef==null || !sourcef.isFile()) {
			mErrorCode = 400;
			mErrorMsg = "Source file not found. File:"+source;
			if (ShareApp.DEBUG) Log.e(TAG, mErrorMsg);
			return;
		}
		else {
			if (ShareApp.DEBUG) Log.e(TAG, "Source file found. File:"+source);
		}
		
		try {
			Scanner sc = new Scanner(sourcef);
			sc.next();	sc.next();	sc.next();
			String s4 = sc.next();
			if (	s4.contains(MyApp.PACKAGE_PROA) || s4.contains(MyApp.PACKAGE_LITE) || 
					s4.contains(MyApp.PACKAGE_DEMO) || s4.contains(MyApp.PACKAGE_ADVR)) {
				if (ShareApp.DEBUG) Log.e(TAG, "Good! signature found:"+s4);
			}
			else {
				mErrorCode = 401;
	        	mErrorMsg = "Source is not right backup file. No signature found! Source header: "+s4;
				if (ShareApp.DEBUG) Log.e(TAG, mErrorMsg);
				return;
			}
		}
		catch (Exception e) {
			mErrorCode = 402;
        	mErrorMsg = "Exception occured while reading file "+e.getMessage();
			return;
		}
		
		// Empty the target tables.
		String[] tables = DBAdapter.TABLES_IN_USE;
		mDbHelper.rebuildTables(tables);
		if (MyApp.DEBUG) Log.e(TAG,"rebuilding tables. ");
		
		// Import the data from datafile
		try {
			XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
			XmlPullParser xpp = factory.newPullParser();
			
			File f = new File(source);
			InputStream is = new FileInputStream(f);
			xpp.setInput(is,"utf-8");
			
			EntrData mNote = new EntrData();
			ItemData mItem = new ItemData();
			LblmData mLblm = new LblmData();
			LbliData mLbli = new LbliData();
			TaskData mTask = new TaskData();
			Lb2mData mLb2m = new Lb2mData();
			
			String currentTable="";
			String fieldName="";
			while(xpp.getEventType()!=XmlPullParser.END_DOCUMENT) {
				// setting default values
				if (xpp.getEventType()==XmlPullParser.START_TAG) {
					if (xpp.getName().equals("table")) {
						currentTable=xpp.getAttributeValue(0);
					}
					else if (xpp.getName().equals("row")) {
						if (currentTable.equals(tables[0])) 		{	mNote = new EntrData();	}
						else if (currentTable.equals(tables[1])) 	{	mItem = new ItemData();	}
						else if (currentTable.equals(tables[2])) 	{	mLblm = new LblmData();	}
						else if (currentTable.equals(tables[3])) 	{	mLbli = new LbliData();	}
						else if (currentTable.equals(tables[4])) 	{	mTask = new TaskData();	}
						else if (currentTable.equals(tables[5])) 	{	mLb2m = new Lb2mData();	}
					}
					else if (xpp.getName().equals("col")) {
						fieldName = xpp.getAttributeValue(0);
					}
				}
				// Getting field data
				else if (xpp.getEventType()==XmlPullParser.TEXT) {
					if (currentTable.equals(tables[0])) 		{	mNote.setData(fieldName,xpp.getText());	}
					else if (currentTable.equals(tables[1])) 	{	mItem.setData(fieldName,xpp.getText());	}
					else if (currentTable.equals(tables[2])) 	{	mLblm.setData(fieldName,xpp.getText());	}
					else if (currentTable.equals(tables[3])) 	{	mLbli.setData(fieldName,xpp.getText());	}
					else if (currentTable.equals(tables[4])) 	{	mTask.setData(fieldName,xpp.getText());	}
					else if (currentTable.equals(tables[5])) 	{	mLb2m.setData(fieldName,xpp.getText());	}
				}
				// Saving the row
				else if (xpp.getEventType()==XmlPullParser.END_TAG) {
					if (xpp.getName().equals("row")) {
						long x = -1L;
						if (currentTable.equals(tables[0])) 		{	x = mNote.createRecord(mDbHelper);	}
						else if (currentTable.equals(tables[1])) 	{	x = mItem.createRecord(mDbHelper);	}
						else if (currentTable.equals(tables[2])) 	{	x = mLblm.createRecord(mDbHelper);	}
						else if (currentTable.equals(tables[3])) 	{	x = mLbli.createRecord(mDbHelper);	}
						else if (currentTable.equals(tables[4])) 	{	x = mTask.createRecord(mDbHelper);	}
						else if (currentTable.equals(tables[5])) 	{	x = mLb2m.createRecord(mDbHelper);	}
					
						if (mBackupMode==BACKUP_MODE_ACTIVITY) {
							if (x>0)	insertCount++;
							if (mContext instanceof PrefBackupLocal)
								((PrefBackupLocal) mContext).setProgressMessage("Restoring database\nRecord imported: "+insertCount);
							else
								((PrefBackupDrbx) mContext).setProgressMessage("Restoring database\nRecord imported: "+insertCount);
						}
					}
				}
				
				xpp.next();
			}
			mErrorCode = 0;
        	mErrorMsg = "Database restored successfully";
		}
		catch (Throwable t) {
			t.printStackTrace();
			mErrorCode = 403;
        	mErrorMsg = "Exception occured while restoring database."+t.toString();
		}
		
		mDbHelper.close();
	}
	
	
	
}
