package com.acj0.orangediaryproa.mod.backup;

import java.io.File;

import android.app.Service;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

import com.acj0.orangediaryproa.data.DBAdapter;
import com.acj0.orangediaryproa.data.MyApp;
import com.acj0.orangediaryproa.data.Notification1;
import com.acj0.share.ShareIntent;
import com.acj0.share.SharePref;
import com.acj0.share.mod.api.dropbox.Dropbox;
import com.acj0.share.utils.IOUtils;
import com.acj0.share.utils.LogUtils;
import com.acj0.share.utils.Util;
import com.acj0.share.utils.ZipUtil;

public class BackupService extends Service {
    
	// This module is completely transferrable 2013-12-06
	private static final String TAG = "BackupService"; 

	// MV
	private int mExtraBackupType = Backup10.BACKUP_TYPE_SCHEDL;
	private boolean mExtraShowNotif = false;
	private boolean mExtraForceFullOnline = false;
	
	private Backup10 mBackup;
	private LogUtils mLog;
	
	// Preferences
	private SharedPreferences mSharedPref;
	//private SharedPreferences.Editor mSharedPrefEdit;
	
    @Override
    public void onCreate() {
        if (MyApp.DEBUG)	Log.e(TAG,"onCreate");
    }
    
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		
		if (MyApp.DEBUG) Log.e(TAG, "flag: "+flags);
		if (MyApp.DEBUG) Log.e(TAG, "startId: "+startId);
		
		// Extras
		// http://stackoverflow.com/questions/4679654/unable-to-start-service-service-name-with-null
		if (intent!=null) {
			Bundle extras = intent.getExtras();
			if (extras != null) {
				mExtraBackupType = extras.getInt(ShareIntent.EXTRA_BACKUP_TYPE);
				mExtraShowNotif = extras.getBoolean(ShareIntent.EXTRA_BACKUP_SHOW_NOTIF);
				mExtraForceFullOnline = extras.getBoolean(ShareIntent.EXTRA_BACKUP_FORCE_FULL);
				if (mExtraShowNotif) showNotification(0, "");
			}
		}
		
		// MV
		mBackup = new Backup10(this, Backup10.BACKUP_MODE_SERVICE);
        mSharedPref = PreferenceManager.getDefaultSharedPreferences(this);
		
		mLog = new LogUtils(MyApp.BASE_DIR);
		mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Start by "+Backup10.BACKUP_TYPE_ABBR[mExtraBackupType]);
		
		// Go or No-go
		if (mExtraBackupType==Backup10.BACKUP_TYPE_SCHEDL) {
			long latest = mBackup.getLatestBackupTime(0);
			long now = System.currentTimeMillis();

			String timeInterval = mSharedPref.getString(SharePref.KEY_BACKUP_INTERVAL, SharePref.VAL_BACKUP_INTERVAL);
			int timeIntervalI = Util.parseInt(timeInterval);
			long timeIntervalL = Backup10.TIME_INTERVAL[timeIntervalI];
			if (latest+(timeIntervalL*.5)<now) {
				Thread thread = new Thread(null, mTask, "BackupService"); 
				thread.start(); 
			}
			else {
				long duration = now - latest;
				boolean withSecond = false;
				String dur = Util.getDurationString(duration, withSecond);
				
				mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: "+dur+" since last backup");
				mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Backup Skipped\n");
				stopSelf();
			}
		}
		else {
			Thread thread = new Thread(null, mTask, "BackupService"); 
			thread.start(); 
		}
		
		return super.onStartCommand(intent, flags, startId);
	}

	@Override
    public void onDestroy() {
		if (MyApp.DEBUG)	Log.e(TAG,"onDestroy");
    }

	@Override
	public IBinder onBind(Intent arg0) {
 		return null;
	}

	
	private Runnable mTask = new Runnable() { 
		public void run() { 
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Backup starting");
			
			String now = Backup10.SDF.format(System.currentTimeMillis());
			String extension = ".msz";
			if (mExtraBackupType==Backup10.BACKUP_TYPE_LATEST) {
				extension = ".msa";
			}
			
			File fMsz = new File(MyApp.BACKUP_DIR, now+extension);
			File fDat = new File(MyApp.BACKUP_DIR, now+"."+DBAdapter.DATABASE_NAME);
			File fXml = new File(MyApp.BACKUP_DIR, now+".xml");
			File fInf = new File(MyApp.BACKUP_DIR, now+".inf");
			
	    	mBackup.backupDat1(fDat.getAbsolutePath());

    		if (mBackup.mErrorCode==0) {
	    		mBackup.backupXml(fXml.getAbsolutePath());
	        	
	        	if (mBackup.mErrorCode==0) {
	        		
	        		mBackup.createSysInfoFile(fInf);
	        		
		        	String[] files = {
		        			fInf.getAbsolutePath(), 
		        			fDat.getAbsolutePath(), 
		        			fXml.getAbsolutePath(), 
		        	};
		        	
		        	ZipUtil.zipFile(fMsz, files);
		        	IOUtils.rm(fInf);
		    		IOUtils.rm(fDat);
		    		IOUtils.rm(fXml);
		    		
		    		if (fMsz.isFile()) {
		    			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Backup file created");
		    			backupXMLOnline(fMsz);
		    		}
		    		else {
		    			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Error file not created");
		    			IOUtils.rm(fInf);
			    		IOUtils.rm(fDat);
			    		IOUtils.rm(fXml);
		    		}
		    		
		    	}
	        	else {
		    		IOUtils.rm(fDat);
	        	}
	    	}
    		
    		// remove old backups
   			int deleteCount = mBackup.deleteOldBackups(mExtraBackupType);
        	if (deleteCount>0) {
    			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Removed old backups: "+deleteCount);
    		}
        		
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Backup completed\n");
			if (mExtraShowNotif) { 
				showNotification(3, "");
			}
			
			stopSelf();
			
		}
	};
	
	public void backupXMLOnline(File fMsz) {
		
		int prefBackupOnline = mSharedPref.getInt(SharePref.KEY_BACKUP_ONLINE, SharePref.VAL_BACKUP_ONLINE);
		if (mExtraForceFullOnline)	{
			prefBackupOnline = 2;
		}
		
		if (prefBackupOnline==0) {
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Online backup disabled");
			return;
		}
		
		Dropbox dropbox = new Dropbox(this);
		dropbox.processAuthentication();
			
		if (!dropbox.mLoggedIn) {
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Online backup disabled. unlink");
			return;
		}
		
		if (mExtraShowNotif) showNotification(1, "");
		String fileRemote = Dropbox.BACKUP_DIR+fMsz.getName();
		dropbox.putFile(fMsz.getAbsolutePath(), fileRemote);
		
		if (dropbox.mErrorCode==0) {
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Online backup completed");
		}
		else {
			mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Error in Online backup\n\n"+
					dropbox.mErrorCode+". "+dropbox.mErrorMessage+"\n");
		}
		
		// photo & voice sync
		if (prefBackupOnline==2) {
			for (int i=0; i<Backup10.DROPBOX_BACKUP_DIR.length; i++) {
				String folder = Backup10.DROPBOX_BACKUP_DIR[i][1].replaceAll("/", "");
				if (mExtraShowNotif) showNotification(2, "files in "+folder);
				
				dropbox.syncDir(Backup10.DROPBOX_BACKUP_DIR[i][0], Backup10.DROPBOX_BACKUP_DIR[i][1]);
				
				if (dropbox.mErrorCode==0) {
					mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: "+dropbox.mErrorMessage);
				}
				else {
					mLog.logStatus(LogUtils.LOG_BACKUP, "SVC: Error in sync. "+Backup10.DROPBOX_BACKUP_DIR[i][1]+"\n\n"+
							dropbox.mErrorCode+". "+dropbox.mErrorMessage+"\n");
				}
			}
		}
	}
	
	
	private void showNotification(int type, String text) {
		
		String tickerText = MyApp.APP_NAME_LONG+" Backup running";
		String contentTitle = MyApp.APP_NAME_LONG+" Online backup";
		String contentText = "Creating backup files";
		
		if (type==1) {
			tickerText = MyApp.APP_NAME_LONG+" Backup running";
			contentTitle = MyApp.APP_NAME_LONG+" Online backup";
			contentText = "Uploading text data....";
		}
		else if (type==2) {
			tickerText = MyApp.APP_NAME_LONG+" Backup running";
			contentTitle = MyApp.APP_NAME_LONG+" Online backup";
			contentText = "Uploading "+text+"...";
		}
		else if (type==3) {
			tickerText = MyApp.APP_NAME_LONG+" Backup completed";
			contentTitle = MyApp.APP_NAME_LONG+" Online backup";
			contentText = "Completed!";
		}
		
		Notification1 notif = new Notification1(this);
		notif.startNotificationBackup(tickerText, contentTitle, contentText);
		
    }

	

}

