package com.acj0.orangediaryproa.data;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Calendar;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.text.format.DateUtils;
import android.util.Log;

import com.acj0.orangediaryproa.R;
import com.acj0.share.utils.IOUtils;


public class CopyOfDBAdapter {

	private static final String TAG = "DBAdapter";
	
	// Very important! this number should be fixed to 28 2010-07-10
	private static final int DATABASE_VERSION = 28;
	// Very important! this number should be fixed to 28 2010-07-10
	// When need to change, please uncomment "onUpgrade" section below.
	
	public static final String DATABASE_NAME = "data";
	
	// ==================
	// F001: NOTE 
	// ==================
	private static final String DATABASE_CREATE_NOTE =
		"create table note (" +
		"_id integer primary key autoincrement, " +			// 0
		"title text, " +									// 1	
		"body text, " +										// 2
		"created integer, " +								// 3
		"groupid integer, " +								// 4
		"trash text, " +									// 5	
		"remindertime integer, " +							// 6	
		"remindertype String, " +							// 7	
		"ztxt0 text, ztxt1 text, ztxt2 text, ztxt3 text, ztxt4 text, " +
		"ztxt5 text, ztxt6 text, ztxt7 text, ztxt8 text, ztxt9 text, " +
		"zint0 integer, zint1 integer, zint2 integer, zint3 integer, zint4 integer, " +
		"zint5 integer, zint6 integer, zint7 integer, zint8 integer, zint9 integer)";

	public static final String TBL_ENTR = "note"; 
	public static final String FLD_ENTR_ID = "_id";							
	public static final String FLD_ENTR_TITLE = "title";						
	public static final String FLD_ENTR_BODY = "body";						
	public static final String FLD_ENTR_CREATED = "created";					
	public static final String FLD_ENTR_GROUPID = "groupid";					
	public static final String FLD_ENTR_TRASH = "trash";						
	public static final String FLD_ENTR_REMINDER_TIME = "remindertime";
//	public static final String KEY_NOTE_REMINDER_TYPE = "remindertype";		// v20 switched to int
	public static final String FLD_ENTR_REMINDER_TYPE = "zint0";		
	
	public static final String[] PROJ_ENTR = {
		FLD_ENTR_ID,				// 0
		FLD_ENTR_TITLE,				// 1
		FLD_ENTR_BODY,				// 2
		FLD_ENTR_CREATED,			// 3
		FLD_ENTR_GROUPID,			// 4
		FLD_ENTR_TRASH,				// 5	"N",normal or "T",trash, "M" template
		FLD_ENTR_REMINDER_TIME,		// 6
		FLD_ENTR_REMINDER_TYPE		// 7	String field: "Notification" or "Alarm" > v20 changed to int 0: notif, 1: alarm
	};
	
    public long createNoteBodyCreatedStar(String title, String body, long created, int groupid, String trash) {
        
    	String table = TBL_ENTR;
    	String nullColumnHack = null;
    	
    	ContentValues values = new ContentValues();
        values.put(FLD_ENTR_TITLE, title);
		values.put(FLD_ENTR_BODY, body);
		values.put(FLD_ENTR_CREATED, created);
		values.put(FLD_ENTR_GROUPID, groupid);
		values.put(FLD_ENTR_TRASH, trash);
		values.put(FLD_ENTR_REMINDER_TIME, 0);
		values.put(FLD_ENTR_REMINDER_TYPE, 0);
		
		return mDb.insert(table, nullColumnHack, values);
    }
    

    public int updateNoteBodyCreatedStar(long id, String title, String body, long created, int groupid) {
    	
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID + "=" + id;
    	String[] whereArgs = null;
    	
        ContentValues values = new ContentValues();
        values.put(FLD_ENTR_TITLE, title);
        values.put(FLD_ENTR_BODY, body);
        values.put(FLD_ENTR_CREATED, created);
        values.put(FLD_ENTR_GROUPID, groupid);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }

    
    public int updateNoteGroupId(long id, int groupid) {
    	
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID + "=" + id;
    	String[] whereArgs = null;

    	ContentValues values = new ContentValues();
    	values.put(FLD_ENTR_GROUPID, groupid);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }

    public int updateNoteCreated(long id, long created) {

    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID + "=" + id;
    	String[] whereArgs = null;

    	ContentValues values = new ContentValues();
    	values.put(FLD_ENTR_CREATED, created);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }

    public int updateNoteTrash(long id, String trash) {
    	// "N": normal, "T": trash, "M": template
    	
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID+"="+id;
    	String[] whereArgs = null;

    	ContentValues values = new ContentValues();
    	values.put(FLD_ENTR_TRASH, trash);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    
    public int updateNoteReminder(long id, int remindertype, long remindertime) {
    	// 0: notif, 1: alarm
    	
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID + "=" + id;
    	String[] whereArgs = null;

    	ContentValues values = new ContentValues();
    	values.put(FLD_ENTR_REMINDER_TIME, remindertime);
    	values.put(FLD_ENTR_REMINDER_TYPE, remindertype);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    
    public int updateNoteTrashByLabel(String label, String trash) {
    	// "N": normal, "T": trash
    	
    	String table = TBL_ENTR;
    	String whereClause = 
    		"_id IN (SELECT noteid FROM lbl WHERE label=? ) ";
    	String[] whereArgs = new String[] {label};
    	
    	ContentValues values = new ContentValues();
    	values.put(FLD_ENTR_TRASH, trash);
    	
		return mDb.update(table, values, whereClause, whereArgs);
    }
	
    public int updateNoteTrashByPeriod(long periodF, long periodT, String trash) {
    	// "N": normal, "T": trash; "M": Template
    	
    	String table = TBL_ENTR;
    	String whereClause = 
    		FLD_ENTR_TRASH+" NOT IN ('T','M') AND "+
			FLD_ENTR_CREATED+">="+periodF+" AND "+
    		FLD_ENTR_CREATED+"<="+periodT;
    	String[] whereArgs = null;
    	
        ContentValues values = new ContentValues();
        values.put(FLD_ENTR_TRASH, trash);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
	
    public int deleteNote(long id) {
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_ID + "=" + id;
    	String[] whereArgs = null;
    	
    	return mDb.delete(table, whereClause, whereArgs);
    }
    
    public int deleteAllNoteByTrash(String trash) {
    	String table = TBL_ENTR;
    	String whereClause = FLD_ENTR_TRASH + "='"+trash+"'";
    	String[] whereArgs = null;
    	
    	return mDb.delete(table, whereClause, whereArgs);
    }
    
    public Cursor fetchNotes(String[] columns) {
    	String table = TBL_ENTR;
//    	String[] columns = null;    	M
    	String selection = null;
    	String[] selectionArgs = null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchAllNote1(String[] columns, String orderBy) {
    	String table = TBL_ENTR;
//    	String[] columns = null;
    	String selection = FLD_ENTR_TRASH+" NOT IN ('T','M')";
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchAllNote2(String[] columns, String orderBy) {
    	String table = TBL_ENTR;
//    	String[] columns = null;
    	String selection = FLD_ENTR_TRASH+" !='M'";
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }

    public Cursor fetchNoteByPeriod(String[] columns, long periodF, long periodT, String orderBy) {
    	String table = TBL_ENTR;
//    	String[] columns = null;    	
    	String selection = 
    		FLD_ENTR_CREATED+">="+periodF+" AND "+
    		FLD_ENTR_CREATED+"<"+periodT+" AND "+
//    		KEY_NOTE_TRASH+"!='T'";
    		FLD_ENTR_TRASH+" NOT IN ('T','M')";
    	String[] selectionArgs = null;
    	String groupBy=null;
    	String having=null;
//    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchAllNoteByPeriod1(String[] columns, long periodT, String orderBy, String limit) {
    	
    	String table = TBL_ENTR;
//    	String[] columns = null;
    	String selection = 
    		FLD_ENTR_CREATED+"<"+periodT+" AND "+
    		FLD_ENTR_TRASH+" NOT IN ('T','M')";
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = null;
//    	String limit = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy, limit);
    }
    

    
//    public Cursor fetchAllNoteWithLimit(String[] columns, String orderBy, String limit) {
//    	String table = TBL_NOTE;
////    	String[] columns = null;
//    	String selection = KEY_NOTE_TRASH+" NOT IN ('T','M')";
//    	String[] selectionArgs = null;
//    	String groupBy = null;
//    	String having = null;
////    	String limit = null;
//    	
//    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy, limit);
//    }
    
    public Cursor fetchAllNoteByTrash(String[] columns, String trash) {
    	String table = TBL_ENTR;
//    	String[] columns = null;
    	String selection = FLD_ENTR_TRASH + "='"+trash+"'";
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	public Cursor fetchNoteByReminder1(String[] columns) {
    	String table = TBL_ENTR;
//    	String[] columns = null;
    	String selection = FLD_ENTR_REMINDER_TIME+ ">"+System.currentTimeMillis();
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    	
    }
	
    public Cursor fetchAllNoteByLabel(String[] columns, String label, String orderBy) {
    	
    	String table = TBL_ENTR;
//    	String[] columns = null;    	
    	String selection = 
    		FLD_ENTR_TRASH + " NOT IN ('T','M') AND "+
    		FLD_ENTR_ID+" IN (" +
    				" SELECT "+KEY_LBLI_ENTR_ID+" FROM "+TBL_LBLI+
    				" WHERE "+FLD_LBLI_LBLM_ID+"=? ) ";
    	String[] selectionArgs = new String[] {label};
    	String groupBy=null;
    	String having=null;
//    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchCountByGroup(String label, int labeltype) {
    	
    	String table = TBL_ENTR;
    	String[] columns = new String[] {
    			CopyOfDBAdapter.FLD_ENTR_GROUPID,
    			"COUNT(*)"
    	};    	
    	
    	String selection = FLD_ENTR_TRASH + " NOT IN ('T','M')";
    	if (labeltype==-1) {				// root
//    		selection += " AND "+FLD_ENTR_CAT_ID+"="+labelid;
    	}
    	else if (labeltype==0) {			// tag
    		selection += " AND "+FLD_ENTR_ID+" IN (" +
			" SELECT "+KEY_LBLI_ENTR_ID+" FROM "+TBL_LBLI+
			" WHERE "+FLD_LBLI_LBLM_ID+"=? ) ";
    	}
    	
    	String[] selectionArgs = null;
    	if (label!=null && label.length()>0)
    		selectionArgs = new String[] {label};
    	
    	String groupBy=CopyOfDBAdapter.FLD_ENTR_GROUPID;
    	String having=null;
    	String orderBy=CopyOfDBAdapter.FLD_ENTR_GROUPID+" ASC";
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }

    
    public Cursor fetchNoteByQuery1(String[] columns, String selection, String[] selectionArgs, String orderBy) {
    	
    	String table = TBL_ENTR;
//    	String[] columns = null;    	
//    	String selection = "body like ? or title like ?";
//    	String[] selectionArgs = new String[] {likeString, likeString};
    	String groupBy=null;
    	String having=null;
//    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchNoteByQuery1(String[] columns, String selection, String[] selectionArgs, String orderBy, String limit) {
    	
    	String table = TBL_ENTR;
//    	String[] columns = null;    	
//    	String selection = "body like ? or title like ?";
//    	String[] selectionArgs = new String[] {likeString, likeString};
    	String groupBy=null;
    	String having=null;
//    	String orderBy=null;
//    	String limit = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy, limit);
    }

    public Cursor fetchNote(String[] columns, long id) {
        String table = TBL_ENTR;
//    	String[] columns = null;    	
    	String selection = FLD_ENTR_ID + "=" + id;
    	String[] selectionArgs = null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
        
    public Cursor fetchNoteWithoutLabel(String[] columns, String orderBy) {
    	
    	String table = TBL_ENTR;
//    	String[] columns = null;    	
    	String selection = 
    		"trash NOT IN ('T','M') AND NOT EXISTS ("+
    	    		"SELECT noteid " +
    	    		" FROM lbl " +
    	    		" WHERE lbl.noteid=note._id " +
    	    		" GROUP BY noteid ) ";
    	String[] selectionArgs = null;
    	String groupBy=null;
    	String having=null;
//    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	public Cursor fetchNotesByReminder1(String[] proj) {
		return	mDb.query(
        			TBL_ENTR, 
        			proj, 
        			FLD_ENTR_REMINDER_TIME+ ">"+System.currentTimeMillis(), 
        			null, null, null, null, null);
    }

    
    // implemented 12/01/09
    public int updateNullTrash() {
        ContentValues cv1 = new ContentValues();
		cv1.put(FLD_ENTR_TRASH, "N");
		return mDb.update(
				TBL_ENTR, 
				cv1, 
				FLD_ENTR_TRASH+" is null ", 
				null);
    }

	
	// ==================
	// F002: NOTE ITEM (MULTIMEDIA ATTACHMENT) 
	// ==================
	private static final String DATABASE_CREATE_ITEM =
		"create table item (" +
		"_id integer primary key autoincrement, " +			// 0
		"noteid integer, " +								// 1	
		"title text, " +									// 2
		"desc1 text, " +									// 3	
		"desc2 text, " +									// 4	
		"seq integer, " +									// 5	
		"ztxt0 text, ztxt1 text, ztxt2 text, ztxt3 text, ztxt4 text, " +
		"ztxt5 text, ztxt6 text, ztxt7 text, ztxt8 text, ztxt9 text, " +
		"zint0 integer, zint1 integer, zint2 integer, zint3 integer, zint4 integer, " +
		"zint5 integer, zint6 integer, zint7 integer, zint8 integer, zint9 integer)";

	public static final String TBL_ITEM = "item"; 
	public static final String FLD_ITEM_ID = "_id";
	public static final String FLD_ITEM_NOTEID = "noteid";		
	public static final String FLD_ITEM_TITLE = "title";
	public static final String FLD_ITEM_FILEURI = "desc1";
//	public static final String FLD_ITEM_CONTENTURI = "desc2";
	public static final String FLD_ITEM_TYPE = "seq";				
	public static final String FLD_ITEM_INT0 = "zint0";			
		

	public static final String[] PROJ_ITEM = {
		CopyOfDBAdapter.FLD_ITEM_ID,			// 0
		CopyOfDBAdapter.FLD_ITEM_TYPE,		// 1 Type (0:photo, 1: image, 2: video, 3: recording, 4: audio, 99: calendar)
		CopyOfDBAdapter.FLD_ITEM_FILEURI,		// 2 File Uri >> affectes Sound,Image note
		CopyOfDBAdapter.FLD_ITEM_TITLE,		// 3
		CopyOfDBAdapter.FLD_ITEM_NOTEID,		// 4 
		CopyOfDBAdapter.FLD_ITEM_INT0,		// 5 
	};
	
	// KEY_ITEM_TYPE: 99
	public static final String[] PROJ_CAL_EVENT = {
		CopyOfDBAdapter.FLD_ITEM_ID,			// 0
		CopyOfDBAdapter.FLD_ITEM_NOTEID, 		// 1	use this for its id (timestamp)
		CopyOfDBAdapter.FLD_ITEM_TITLE,		// 2
		CopyOfDBAdapter.FLD_ITEM_INT0			// 3	use this for color
	};
	
	
    public long createItem(long noteid, String title,  String fileuri, int type) {
    	
    	String table = TBL_ITEM;
    	String nullColumnHack = null;

   		ContentValues values = new ContentValues();
   		values.put(FLD_ITEM_NOTEID, noteid);
   		values.put(FLD_ITEM_TITLE, title);
//   	values.put(KEY_ITEM_COMPLETED, completed);
//   	values.put(KEY_ITEM_DESC1, desc1);
   		values.put(FLD_ITEM_FILEURI, fileuri);
//   		values.put(FLD_ITEM_CONTENTURI, contenturi);
//   	values.put(KEY_ITEM_SEQ, seq);
   		values.put(FLD_ITEM_TYPE, type);
		
		return mDb.insert(table, nullColumnHack, values);
    }

//    public int updateItemContentProvider(long id, String contenturi) {
//    	
//    	String table = TBL_ITEM;
//    	String whereClause = FLD_ITEM_ID + "=" + id;
//    	String[] whereArgs = null;
//    	
//    	ContentValues values = new ContentValues();
//    	values.put(FLD_ITEM_CONTENTURI, contenturi);
//	   		
//	   	return mDb.update(table, values, whereClause, whereArgs);
//    }
    
    public int updateItemFileuri(long id, String fileuri) {
    	
    	String table = TBL_ITEM;
    	String whereClause = FLD_ITEM_ID + "=" + id;
    	String[] whereArgs = null;
    	
    	ContentValues values = new ContentValues();
    	values.put(FLD_ITEM_FILEURI, fileuri);
	   		
	   	return mDb.update(table, values, whereClause, whereArgs);
    }
    
	public int deleteItem(long itemid) {
		String table = TBL_ITEM;
		String whereClause = FLD_ITEM_ID+"="+itemid;
		String[] whereArgs = null;
		
		return mDb.delete(table, whereClause, whereArgs);
	}
    
   public int deleteItemByNoteid(long noteid) {
		String table = TBL_ITEM;
		String whereClause = FLD_ITEM_NOTEID+"="+noteid;
		String[] whereArgs = null;
		
		return mDb.delete(table, whereClause, whereArgs);
    }

    public int deleteItemByTrash() {
    	String table = TBL_ITEM;
	    String whereClause = 
	    	FLD_ITEM_NOTEID+" IN (" +
	    		"SELECT "+FLD_ENTR_ID+
	    		" FROM "+TBL_ENTR+
	    		" WHERE "+FLD_ENTR_TRASH+"='T') ";
	    String[] whereArgs = null;
	       
	    return mDb.delete(table, whereClause, whereArgs);
    }
    
    public Cursor fetchItems() {
 	   
    	String table = TBL_ITEM;
    	String[] columns = null;    	
    	String selection = null;
    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }

    public Cursor fetchItemByType(String[] columns, String type) {
  	   
    	String table = TBL_ITEM;
//		String[] columns = null;    	
    	String selection = FLD_ITEM_TYPE+" IN ("+type+")";
    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchItemByNoteid(String[] columns, long noteid) {
 	   
    	String table = TBL_ITEM;
//		String[] columns = null;    	
    	String selection = FLD_ITEM_NOTEID+"="+noteid;
    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchItemByNoteidType(String[] columns, long noteid, String type, String orderBy) {
   	   
   	   String table = TBL_ITEM;
//   	   String[] columns = null;    	
   	   String selection = FLD_ITEM_NOTEID+"="+noteid+" AND "+FLD_ITEM_TYPE+" IN ("+type+")";
   	   String[] selectionArgs=null;
   	   String groupBy=null;
   	   String having=null;
   	   if (orderBy==null)	  orderBy = FLD_ITEM_TYPE+" ASC";
      	
   	   return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
      	
	}
    
    public Cursor fetchItemByTrash(String[] columns, String trash) {
  	   
    	String table = TBL_ITEM;
//		String[] columns = null;    	
    	String selection = 
	    	FLD_ITEM_NOTEID+" IN (" +
	    		"SELECT "+FLD_ENTR_ID+
	    		" FROM "+TBL_ENTR+
	    		" WHERE "+FLD_ENTR_TRASH+"='"+trash+"') ";
    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	public Cursor fetchItemSpecial001(String[] columns, String type, String[] selectionArgs) {
	 	   
    	String table = TBL_ITEM;
//		String[] columns = null;    	
    	String selection = FLD_ITEM_TYPE+" IN ("+type+") AND "+FLD_ITEM_FILEURI+" NOT LIKE ? ";
//    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
    	String orderBy=null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    // ==================================
    // for calendar event
    // ==================================
    
    public long createCalEvent(long noteid, String title, int color) {

	   	String table = TBL_ITEM;
	   	String nullColumnHack = null;
	
	   	ContentValues values = new ContentValues();
   		values.put(FLD_ITEM_NOTEID, noteid);
   		values.put(FLD_ITEM_TITLE, title);
//   	values.put(KEY_ITEM_COMPLETED, completed);
//   	values.put(KEY_ITEM_DESC1, desc1);
   		values.put(FLD_ITEM_FILEURI, "");
//   		values.put(FLD_ITEM_CONTENTURI, "");
//   	values.put(KEY_ITEM_SEQ, seq);
   		values.put(FLD_ITEM_TYPE, 99);
		values.put(FLD_ITEM_INT0, color);
		
	   	return mDb.insert(table, nullColumnHack, values);
	}
	
	public int updateCalEvent(long noteid, String title, int color) {

	   	String table = TBL_ITEM;
		String whereClause = 
			FLD_ITEM_NOTEID + "=" + noteid+" AND "+
			FLD_ITEM_TYPE+"=99";
		String[] whereArgs = null;
	
		ContentValues values = new ContentValues();
		values.put(FLD_ITEM_TITLE, title);
		values.put(FLD_ITEM_INT0, color);
	   		
   		return mDb.update(table, values, whereClause, whereArgs);
	}
   

    public int deleteCalEvent(long noteid) {
    	String table = TBL_ITEM;
    	String whereClause = 
    		FLD_ITEM_NOTEID+"="+noteid+" AND "+
    		FLD_ITEM_TYPE+"=99";
    	String[] whereArgs = null;
	       
    	return mDb.delete(table, whereClause, whereArgs);
    }
    
	public Cursor fetchAllCalEventByPeriod(String[] columns, long periodF, long periodT, String orderBy) {
    	
    	String table = TBL_ITEM;
// 	   	String[] columns = null;    	
    	String selection = 
    		FLD_ITEM_NOTEID+">="+periodF+" AND "+
    		FLD_ITEM_NOTEID+"<="+periodT+" AND "+
    		FLD_ITEM_TYPE+"=99";
    	String[] selectionArgs=null;
    	String groupBy=null;
    	String having=null;
//    	String orderBy="seq ASC";
    	
 	   return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	// ==================
	// l001: LABEL MASTER
	// ==================
	private static final String DATABASE_CREATE_LBLM =
		"create table lbm (" +
		"label text primary key,  " +									// 0
		"ztxt1 text, ztxt2 text, ztxt3 text, ztxt4 text, ztxt5 text, " +
		"zint1 integer, zint2 integer, zint3 integer, zint4 integer, zint5 integer)";

	// only one field is used. 
	public static final String TBL_LBLM = "lbm"; 
	public static final String FLD_LBLM_LABEL = "label";
	public static final String FLD_LBLM_MARKERID = "zint1";
	
	public static final String[] PROJ_LBLM = {
		CopyOfDBAdapter.FLD_LBLM_LABEL,			// 0
		CopyOfDBAdapter.FLD_LBLM_MARKERID, 		// 1	// default marker for the tag
	};
	// Here the concept of no marker (value -1) is important
	
	
    public long createLabelM(String label) {
    	String table = TBL_LBLM;
    	String nullColumnHack = null;
    	
		ContentValues values = new ContentValues();
		values.put(FLD_LBLM_LABEL, label);
		values.put(FLD_LBLM_MARKERID, -1);
		return  mDb.insert(table, nullColumnHack, values);
    }

    public int updateLabelM(String oldS, String newS) {
    	
    	String table=TBL_LBLM;
    	String whereClause=FLD_LBLM_LABEL + "= ? ";
    	String[] whereArgs = new String[] {oldS};
    	
    	ContentValues values = new ContentValues();
    	values.put(FLD_LBLM_LABEL, newS);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    
    public int updateLabelM1(String label, int markerid) {
    	
    	String table=TBL_LBLM;
    	String whereClause=FLD_LBLM_LABEL + "= ? ";
    	String[] whereArgs = new String[] {label};
    	
    	ContentValues values = new ContentValues();
    	values.put(FLD_LBLM_MARKERID, markerid);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    

    public int deleteLabelM(String label) {
    	
    	String table=TBL_LBLM;
    	String whereClause=FLD_LBLM_LABEL + "= ? ";
    	String[] whereArgs = new String[] {label};
    	
		return mDb.delete(table, whereClause, whereArgs);
    }

    public Cursor fetchLabelMAll() {
    	
    	String table = TBL_LBLM;
    	String[] columns = PROJ_LBLM;
        String selection = null;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = "label ASC";
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
	
    public Cursor fetchLabelM(String label) {
    	
    	String table = TBL_LBLM;
    	String[] columns = PROJ_LBLM;
        String selection = FLD_LBLM_LABEL+"=? ";
    	String[] selectionArgs = new String[] {label};
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchLabelMSpecial() {
    	
    	String table = TBL_LBLM;
    	String[] columns = new String[] { FLD_LBLI_LBLM_ID };
    	
    	String selection = FLD_LBLM_LABEL+" LIKE '%,%' ";
    	String[] selectionArgs = null;
    	String groupBy = FLD_LBLM_LABEL;
    	String having=null;
    	String orderBy = null;
    	
    	return	mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	
	// ==================
	// l002: LABEL ITEM, LABEL-NOTEID
	// ==================
	private static final String DATABASE_CREATE_LBLI =
		"create table lbl (" +
		"noteid integer NOT NULL, " +								// 0	// all the same use 0
		"label text NOT NULL, " +									// 1
		"ztxt1 text, ztxt2 text, ztxt3 text, ztxt4 text, ztxt5 text, " +
		"zint1 integer, zint2 integer, zint3 integer, zint4 integer, zint5 integer, " +
		"primary key (noteid, label) )";

	// only two fields are used.
	public static final String TBL_LBLI = "lbl"; 
	public static final String KEY_LBLI_ENTR_ID = "noteid";
	public static final String FLD_LBLI_LBLM_ID = "label";
	public static final String[] PROJ_LBLI = {
		CopyOfDBAdapter.KEY_LBLI_ENTR_ID, 			// 1	
		CopyOfDBAdapter.FLD_LBLI_LBLM_ID,			// 0
	};
	
    public long createLabelI(long noteid, String label) {
		String table = TBL_LBLI;
    	String nullColumnHack = null;
    	
		ContentValues values = new ContentValues();
		values.put(KEY_LBLI_ENTR_ID, noteid);
		values.put(FLD_LBLI_LBLM_ID, label);
		
		
		return mDb.insert(table, nullColumnHack, values);
    }

    public int updateLabelI(String oldLabel, String newLabel) {
    	String table=TBL_LBLI;
    	String whereClause=FLD_LBLI_LBLM_ID+"=? ";
    	String[] whereArgs = new String[] {oldLabel};
    	
		ContentValues values = new ContentValues();
    	values.put(FLD_LBLI_LBLM_ID, newLabel);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    
    public int updateLabelIByNoteIdLabelM(long noteid, String oldLabel, String newLabel) {
    	
    	String table=TBL_LBLI;
    	String whereClause=KEY_LBLI_ENTR_ID+"="+noteid+" AND "+FLD_LBLI_LBLM_ID+"=? ";
    	String[] whereArgs = new String[] {oldLabel};
    	
    	ContentValues values = new ContentValues();
    	values.put(FLD_LBLI_LBLM_ID, newLabel);
		
		return mDb.update(table, values, whereClause, whereArgs);
    }
    

    public int deleteLabelIByEntryId(long noteid) {
        return mDb.delete(
        		TBL_LBLI, 
        		KEY_LBLI_ENTR_ID + "=" + noteid, 
        		null);
    }
    
    public int deleteLabelIByLabelM(String label) {
        return mDb.delete(
        		TBL_LBLI, 
        		FLD_LBLI_LBLM_ID+"=?",
				new String[] {label});
    }

    public int deleteLabelIByNoteidLabel(long noteid, String label) {
    	String table=TBL_LBLI;
    	String whereClause=
    		KEY_LBLI_ENTR_ID+"="+noteid+" AND "+FLD_LBLI_LBLM_ID+"=? ";
    	String[] whereArgs = new String[] {label};
    	
    	return mDb.delete(table, whereClause, whereArgs);
    }
    
    public int deleteLabelIByTrash(String trash) {
    	String table = TBL_LBLI;
    	String whereClause = 
    		KEY_LBLI_ENTR_ID+" IN (" +
	       		"SELECT _id FROM note " +
	       		"WHERE " +
	       			FLD_ENTR_TRASH+"='"+trash+"') ";
	       String[] whereArgs = null;
	       
	       return mDb.delete(table, whereClause, whereArgs);
    }
    
    public int deleteLabelIByPeriod(long periodF, long periodT, String trash) {
    	String table = TBL_LBLI;
    	String whereClause = 
    		KEY_LBLI_ENTR_ID+" IN (" +
       			"SELECT _id FROM note " +
       			"WHERE "+
       				FLD_ENTR_TRASH+" NOT IN ('T','M') AND "+
	 	       		FLD_ENTR_CREATED+">="+periodF+" AND "+
	        		FLD_ENTR_CREATED+"<="+periodT+
       			") ";
 	       String[] whereArgs = null;
 	       
 	      return mDb.delete(table, whereClause, whereArgs);
     }
    
    public Cursor fetchLabelIAll() {
    	
    	String table = TBL_LBLI;
    	String[] columns = null;
    	String selection = null;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
        return	mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }

    public Cursor fetchLabelIByNoteid(long noteid) {
    	
    	String table = TBL_LBLI;
    	String[] columns = PROJ_LBLI;
    	String selection = KEY_LBLI_ENTR_ID+"="+noteid;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
        return	mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchLabelIByLabelM(String label) {
    	
    	String table = TBL_LBLI;
    	String[] columns = new String[] { KEY_LBLI_ENTR_ID };
    	
    	String selection = FLD_LBLI_LBLM_ID+"=? ";
    	String[] selectionArgs = new String[] {label};
    	String groupBy = null;
    	String having=null;
    	String orderBy = null;
    	
    	return	mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
	// ==================
	// L004: TODO ITEM 
	// ==================
	//private static final String DATABASE_TABLE_???? = "ztbl1"; 
	private static final String DATABASE_CREATE_TASK =
		"create table ztbl1 (" +
		"_id integer primary key autoincrement, " +			// 0
		"ztxt1  text, ztxt2  text, ztxt3  text, ztxt4  text, ztxt5  text, "+
		"ztxt6  text, ztxt7  text, ztxt8  text, ztxt9  text, ztxt10 text, "+
		"ztxt11 text, ztxt12 text, ztxt13 text, ztxt14 text, ztxt15 text, "+
		"zint1  integer, zint2  integer, zint3  integer, zint4  integer, zint5  integer, " +
		"zint6  integer, zint7  integer, zint8  integer, zint9  integer, zint10 integer, " +
		"zint11 integer, zint12 integer, zint13 integer, zint14 integer, zint15 integer)";

	public static final String TBL_TASK = 				"ztbl1"; 
	public static final String FLD_TASK_ID = 			"_id";
	public static final String FLD_TASK_TITLE = 		"ztxt1";
	
	public static final String FLD_TASK_COMPLETED = 	"zint1";	// 
	public static final String FLD_TASK_SEQ = 			"zint2";	// 
	public static final String FLD_TASK_LISTID = 		"zint3";	
	public static final String FLD_TASK_DUE = 			"zint4";
	public static final String FLD_TASK_RECTYPE = 		"zint6";	// 
	public static final String FLD_TASK_REMINDER_TIME = 	"zint5";
	public static final String FLD_TASK_REMINDER_TYPE = 	"zint7";
	
	public static final int KEY_TASK_RECTYPE_TASK_HEADER = 	0;
	public static final int KEY_TASK_RECTYPE_TASK_ITEM = 	1;
	public static final int KEY_TASK_RECTYPE_EXPN_HEADER = 	2;
	public static final int KEY_TASK_RECTYPE_EXPN_ITEM = 	3;
	
	public static final String[] PROJ_TASK = {
		FLD_TASK_ID,				// 0
		FLD_TASK_TITLE,				// 1
		FLD_TASK_REMINDER_TYPE,		// 2
		FLD_TASK_COMPLETED,			// 3 0: not yet, 1: completed
		FLD_TASK_SEQ,				// 4 start from 0 . not used from v15
		FLD_TASK_LISTID,			// 5 item use this for its header id
		FLD_TASK_DUE,				// 6
		FLD_TASK_REMINDER_TIME,		// 7 
		FLD_TASK_RECTYPE			// 8 0: header, 1: detail, 2: expense header, 3: expense detail.
	};
	
	// FLD_TODO_RECTYPE: 0
	public static final String[] PROJ_TASK_HEADER = {
		FLD_TASK_ID,				// 0
		FLD_TASK_TITLE,				// 1
		FLD_TASK_DUE,				// 2
		FLD_TASK_REMINDER_TYPE,		// 3	// reminder
		FLD_TASK_REMINDER_TIME,		// 4
	};
	
	// FLD_TODO_RECTYPE: 1
	public static final String[] PROJ_TASK_ITEM = {
		FLD_TASK_ID,				// 0
		FLD_TASK_TITLE,				// 1
		FLD_TASK_REMINDER_TYPE,		// 2
		FLD_TASK_COMPLETED,			// 3
		FLD_TASK_SEQ,				// 4
		FLD_TASK_LISTID,			// 5
		FLD_TASK_DUE,				// 6
		FLD_TASK_REMINDER_TIME		// 7 
	};
	
	// FLD_TODO_RECTYPE: 0
	public static final String[] PROJ_EXPN_HEADER = {
		FLD_TASK_ID,				// 0
		FLD_TASK_TITLE,				// 1
		FLD_TASK_DUE				// 2
	};
	
	// FLD_TODO_RECTYPE: 1
	public static final String[] PROJ_EXPN_ITEM = {
		FLD_TASK_ID,				// 0
		FLD_TASK_TITLE,				// 1
		FLD_TASK_REMINDER_TYPE,		// 2
//		FLD_TASK_COMPLETED,			// 
		FLD_TASK_SEQ,				// 3
		FLD_TASK_LISTID,			// 4
		FLD_TASK_DUE,				// 5
		FLD_TASK_REMINDER_TIME		// 6 
	};
	
	//Standard methods
	public long createTodo(
			String title, int reminderType,
			int completed, int seq, long listId, long due, long reminderTime, int recType) {
		
		String table = TBL_TASK;
		String nullColumnHack = null;
		
		ContentValues values = new ContentValues();
		values.put(FLD_TASK_TITLE, title);
		values.put(FLD_TASK_REMINDER_TYPE, reminderType);
        values.put(FLD_TASK_COMPLETED, completed);
        values.put(FLD_TASK_SEQ, seq);
        values.put(FLD_TASK_LISTID, listId);
        values.put(FLD_TASK_DUE, due);
        values.put(FLD_TASK_REMINDER_TIME, reminderTime);
        values.put(FLD_TASK_RECTYPE, recType);
		
		return mDb.insert(table, nullColumnHack, values);
    }

    public int deleteTodo(long _id) {
    	String table = TBL_TASK;
    	String whereClause = FLD_TASK_ID+"="+_id;
    	String[] whereArgs = null;
    	
    	return mDb.delete(table, whereClause, whereArgs);
    }

    public Cursor fetchTodoAll(String[] columns) {
    	
    	String table = TBL_TASK;
//    	String[] columns = proj;
        String selection = null;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchTodo(String[] columns, long _id) {
    	
    	String table = TBL_TASK;
//    	String[] columns = proj;
        String selection = FLD_TASK_ID + "=" + _id;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
    	String orderBy = null;
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    //Extra methods
	
    public int updateTodoHeader1(long _id, String title) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_TITLE, title);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }
    
    public int updateTodoHeader2(long _id, long due) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_DUE, due);
        return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }
    
    public int updateTodoItem1(long _id, String title, long seq) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_TITLE, title);
        values.put(FLD_TASK_SEQ, seq);
        
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }
    
    public int updateTodoSeqByListid(long headerid, long seq) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_SEQ, seq);
        
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_LISTID+"="+headerid,
				null);
    }
    
    public int updateTodoCreatedByListid(long headerid, long due) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_DUE, due);
        
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_LISTID+"="+headerid,
				null);
    }
    
    public int updateTodoItem2(long _id, int seq) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_SEQ, seq);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }
    
    public int updateTodoItem3(long _id, int completed) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_COMPLETED, completed);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }
    
    public int updateTodoItem31(long headerid, int completed) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_COMPLETED, completed);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_LISTID+"="+headerid,
				null);
    }
    
    public int updateTodoItem4(long _id, long listId) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_LISTID, listId);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }

    public int updateTodoItem5(long _id, long due) {
        ContentValues values = new ContentValues();
        values.put(FLD_TASK_DUE, due);
		return mDb.update(
				TBL_TASK, 
				values, 
				FLD_TASK_ID+"="+_id,
				null);
    }

    public int deleteTodoHeader1(long _id) {
        return mDb.delete(
        		TBL_TASK, 
        		FLD_TASK_ID+"="+_id,
        		null);
    }
    
    public int deleteTodoItem1(long listId) {
        return mDb.delete(
        		TBL_TASK, 
        		FLD_TASK_LISTID+"="+listId,
        		null);
    }

    public Cursor fetchTodoByRectype(String[] columns, int recType, String orderBy) {
    	
    	String table = TBL_TASK;
//    	String[] columns = columns;
        String selection = 
        	FLD_TASK_RECTYPE+"="+recType;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = FLD_TODO_TITLE+" ASC";
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchTodoByRectypeListid(String[] columns, int recType, long listId, String extraSelection, String orderBy) {
    	
    	String table = TBL_TASK;
//    	String[] columns = columns;
        String selection = 
        	FLD_TASK_RECTYPE+"="+recType+" AND "+
        	FLD_TASK_LISTID+"="+listId+
        	extraSelection;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = FLD_TODO_TITLE+" ASC";
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    public Cursor fetchTodoExceptMe(String[] columns, int recType, long _id, String orderBy) {
    	
    	String table = TBL_TASK;
//    	String[] columns = columns;
        String selection = 
        	FLD_TASK_RECTYPE+"="+recType+" AND "+
        	FLD_TASK_ID+"!="+_id;
    	String[] selectionArgs = null;
    	String groupBy = null;
    	String having = null;
//    	String orderBy = FLD_TODO_TITLE+" ASC";
    	
    	return mDb.query(table, columns, selection, selectionArgs, groupBy, having, orderBy);
    }
    
    // ==================
	// F006: ZTBL2 
	// ==================
	//private static final String DATABASE_TABLE_???? = "ztbl2"; 
	
	private static final String DATABASE_CREATE_ZTBL2 =
		"create table ztbl2 (" +
		"_id integer primary key autoincrement, " +		
		"ztxt1  text, ztxt2  text, ztxt3  text, ztxt4  text, ztxt5  text, "+
		"ztxt6  text, ztxt7  text, ztxt8  text, ztxt9  text, ztxt10 text, "+
		"ztxt11 text, ztxt12 text, ztxt13 text, ztxt14 text, ztxt15 text, "+
		"zint1  integer, zint2  integer, zint3  integer, zint4  integer, zint5  integer, " +
		"zint6  integer, zint7  integer, zint8  integer, zint9  integer, zint10 integer, " +
		"zint11 integer, zint12 integer, zint13 integer, zint14 integer, zint15 integer)";
	
	public static final String TBL_ZTB2 = 			"ztbl2"; 
//	public static final String FLD_ZTB2_ID = 			"_id";
	
    public static final String[] DATABASE_TABLES= {
    	CopyOfDBAdapter.TBL_ENTR,		// 0
		CopyOfDBAdapter.TBL_ITEM,		// 1
		CopyOfDBAdapter.TBL_LBLM,		// 2
		CopyOfDBAdapter.TBL_LBLI,		// 3
		CopyOfDBAdapter.TBL_TASK,		// 4		
		CopyOfDBAdapter.TBL_ZTB2,		// 5		
	}; 
    
    public static final String[] TABLES_IN_USE= {
		CopyOfDBAdapter.TBL_ENTR,		// 0
		CopyOfDBAdapter.TBL_ITEM,		// 1
		CopyOfDBAdapter.TBL_LBLM,		// 2
		CopyOfDBAdapter.TBL_LBLI,		// 3
		CopyOfDBAdapter.TBL_TASK,		// 4
	}; 
	
	private DatabaseHelper mDbHelper;
    public SQLiteDatabase mDb;

    private static class DatabaseHelper extends SQLiteOpenHelper {

    	private Context mContext;
        DatabaseHelper(Context context) {
            super(context, DATABASE_NAME, null, DATABASE_VERSION);
            mContext = context;
        }

        @Override
        public void onCreate(SQLiteDatabase db) {
            db.execSQL(DATABASE_CREATE_NOTE);
            db.execSQL(DATABASE_CREATE_ITEM);
            db.execSQL(DATABASE_CREATE_LBLI);
            db.execSQL(DATABASE_CREATE_LBLM);
            db.execSQL(DATABASE_CREATE_TASK);
            db.execSQL(DATABASE_CREATE_ZTBL2);
            addSampleData(db, mContext);
        }

        @Override
        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
            //Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
            //        + newVersion + ", which will destroy all old data");
        	
        	// When need to change, please uncomment onUpgrade section below.
//            db.execSQL("DROP TABLE IF EXISTS note");
//            db.execSQL("DROP TABLE IF EXISTS item");
//            db.execSQL("DROP TABLE IF EXISTS lbl");
//            db.execSQL("DROP TABLE IF EXISTS lbm");
//            db.execSQL("DROP TABLE IF EXISTS ztbl1");
//            db.execSQL("DROP TABLE IF EXISTS ztbl2");
//            onCreate(db);
        	
        	
        	
        	
        	
        }
    
        public boolean importDatabase(String copyDb, String currDb) throws Exception {

            // Close the SQLiteOpenHelper so it will commit the created empty database to internal storage.
            close();
            File copy = new File(copyDb);
            File curr = new File(currDb);
            if (copy.exists() && curr.exists()) {
                IOUtils.copyFile(new FileInputStream(copy), new FileOutputStream(curr));
                // Access the copied database so SQLiteHelper will cache it and mark it as created.
                getWritableDatabase().close();
                return true;
            }
            return false;
        }
        
        public boolean exportDatabase(String currDb, String copyDb) throws Exception {

        	// Close the SQLiteOpenHelper so it will commit the created empty database to internal storage.
        	close();
            File curr = new File(currDb);
            File backup = new File(copyDb);
            if (curr.exists()) {
                IOUtils.copyFile(new FileInputStream(curr), new FileOutputStream(backup));
                // Access the copied database so SQLiteHelper will cache it and mark it as created.
//                getWritableDatabase().close();
                return true;
            }
            return false;
        }
    }
    
    public boolean importDatabase1(String copyDbFile, String currDbFile)  throws Exception {
    	
		return mDbHelper.importDatabase(copyDbFile, currDbFile);
    }
    
    public boolean exportDatabase1(String currDb, String copyDb)  throws Exception {
    	
		return mDbHelper.exportDatabase(currDb, copyDb);
    }
    
    /**
     * Constructor - takes the context to allow the database to be
     * opened/created
     * 
     * @param ctx the Context within which to work
     */
    private final Context mCtx;

	public CopyOfDBAdapter(Context ctx) {
		this.mCtx = ctx;
	}

    /**
     * Open the notes database. If it cannot be opened, try to create a new
     * instance of the database. If it cannot be created, throw an exception to
     * signal the failure
     * 
     * @return this (self reference, allowing this to be chained in an
     *         initialization call)
     * @throws SQLException if the database could be neither opened or created
     */
    public CopyOfDBAdapter open() throws SQLException {
        mDbHelper = new DatabaseHelper(mCtx);
        mDb = mDbHelper.getWritableDatabase();
        return this;
    }
    
    public void close() {
        mDbHelper.close();
    }	
	

    
    
    /**
     * =================================================================================
     * TABLE TODO
     * ================================================================================= 
     */  
      

    
    


    


    // COnsistency check 
    
    public Cursor fetchItemWithoutParent() {
    	String sql = 
    		"SELECT noteid " +
    		" FROM item " +
    		" WHERE seq!=99 " +
    		" GROUP BY noteid " +
    		" EXCEPT " +
    		"SELECT _id " +
    		" FROM note ";
       	return mDb.rawQuery(sql,null);
    }
    
    public Cursor fetchLabelWithoutParent() {
    	String sql = 
    		"SELECT noteid " +
    		" FROM lbl " +
    		" GROUP BY noteid " +
    		" EXCEPT " +
    		"SELECT _id " +
    		" FROM note ";
       	return mDb.rawQuery(sql,null);
    }
    
    
    
    
    // =======================================================
    // Setup sample data
    // =======================================================
    public static void addSampleData(SQLiteDatabase db, Context context) {
        
//    	final String[] title = context.getResources().getStringArray(R.array.sample_data_title);
//    	final String[] body = context.getResources().getStringArray(R.array.sample_data_body);
    	
    	final String[] title = {
    			context.getString(R.string.sample_title_08),
    			context.getString(R.string.sample_title_07),
    			context.getString(R.string.sample_title_06),
    			context.getString(R.string.sample_title_05),
    			context.getString(R.string.sample_title_04),
    			context.getString(R.string.sample_title_03),
    			context.getString(R.string.sample_title_02),
    			context.getString(R.string.sample_title_01),
    	};

    	final String[] body = {
    			context.getString(R.string.sample_data_08),
    			context.getString(R.string.sample_data_07),
    			context.getString(R.string.sample_data_06),
    			context.getString(R.string.sample_data_05),
    			context.getString(R.string.sample_data_04),
    			context.getString(R.string.sample_data_03),
    			context.getString(R.string.sample_data_02),
    			context.getString(R.string.sample_data_01),
    	};
    	
    	String noteInsertPrefix = "INSERT INTO note (_id,title,body,created,groupid,trash,remindertime,remindertype) VALUES ";
    	String labelInsertPrefix = "INSERT INTO lbl (noteid,label) VALUES ";
    	String mlabelInsertPrefix = "INSERT INTO lbm (label,zint1) VALUES ";
    	String caleventPrefix = "INSERT INTO item (noteid,title,desc1,desc2,seq,zint0) VALUES ";
    	

//    	long day = 1000*60*60*24;
    	long day = DateUtils.DAY_IN_MILLIS;
    	long time = System.currentTimeMillis();
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 999);
		long time1 = cal.getTimeInMillis();
		
    	String[] mDatabaseInit = {
    			
				noteInsertPrefix+"(1,'"+title[0]+"','"+body[0]+"',"+(time-day*6)+",81,'N',0,0);",
				noteInsertPrefix+"(2,'"+title[1]+"','"+body[1]+"',"+(time-day*6)+",56,'N',0,0);",
				noteInsertPrefix+"(3,'"+title[2]+"','"+body[2]+"',"+(time-day*4)+",81,'N',0,0);",
				noteInsertPrefix+"(4,'"+title[3]+"','"+body[3]+"',"+(time-day*3)+",55,'N',0,0);",
				noteInsertPrefix+"(5,'"+title[4]+"','"+body[4]+"',"+(time-day*3)+",54,'N',0,0);",
				noteInsertPrefix+"(6,'"+title[5]+"','"+body[5]+"',"+(time-day*1)+",24,'N',0,0);",
				noteInsertPrefix+"(7,'"+title[6]+"','"+body[6]+"',"+(time+day*2)+",24,'N',0,0);",
				noteInsertPrefix+"(8,'"+title[7]+"','"+body[7]+"',"+(time+day*3)+",24,'N',0,0);",
				    			
				labelInsertPrefix+"(1,'Sample Tag 1');",
				labelInsertPrefix+"(2,'Sample Tag 1');",
				labelInsertPrefix+"(3,'Sample Tag 1');",
				labelInsertPrefix+"(4,'Sample Tag 2');",
				labelInsertPrefix+"(5,'Sample Tag 2');",
				labelInsertPrefix+"(6,'Sample Tag 2');",
				labelInsertPrefix+"(7,'Sample Tag 2');",
				labelInsertPrefix+"(8,'Sample Tag 3');",
				labelInsertPrefix+"(4,'Sample Tag 3');",
				labelInsertPrefix+"(4,'Sample Tag 1');",
				labelInsertPrefix+"(8,'Sample Tag 1');",
				
				mlabelInsertPrefix+"('Sample Tag 1',-1);",
				mlabelInsertPrefix+"('Sample Tag 2',-1);",
				mlabelInsertPrefix+"('Sample Tag 3',-1);",
				
				caleventPrefix+"("+(time1+day*0)+",'Sample event','','',99,0);",
				caleventPrefix+"("+(time1+day*-2)+",'Event 1','','',99,1);",
				caleventPrefix+"("+(time1+day*2)+",'Event 2','','',99,2);"


    	};
    	
    	int count = mDatabaseInit.length;
        for (int i=0; i<count; i++) {
        	db.execSQL(mDatabaseInit[i]);
        }
    }
    
    // =======================================================
	// Database Backup and Restore
	// =======================================================
    
    public String getDBPath() {
    	return mDb.getPath();
    }
    
//    public int emptyTable(String table) {
//    	return mDb.delete(	//table, whereClause, whereArgs);
//    			table,
//    			null,
//    			null);
//    }
    
    public void rebuildTables(String[] tables) {
    	
    	int count = tables.length;
    	
    	// drop tables
    	String dropTable = "DROP TABLE IF EXISTS 9999";
    	for (int i=0; i<count; i++) {
    		mDb.execSQL(dropTable.replace("9999", tables[i]));
    		if (MyApp.DEBUG) Log.e(TAG,"deleting "+tables[i]);
    	}
    	
    	// create tables
    	mDb.execSQL(DATABASE_CREATE_NOTE);
    	mDb.execSQL(DATABASE_CREATE_ITEM);
    	mDb.execSQL(DATABASE_CREATE_LBLI);
    	mDb.execSQL(DATABASE_CREATE_LBLM);
    	mDb.execSQL(DATABASE_CREATE_TASK);
		if (MyApp.DEBUG) Log.e(TAG,"creating tables");
    }
    
    public Cursor runRawQuery(String sql) {
       	return mDb.rawQuery(sql,null);
    }
    
    public int countRecord(String table, String selection) {
    	return countRecord(table, selection, null);
    }
    
    public int countRecord(String table, String selection, String[] selectionArgs) {
    	
    	String sql = "SELECT COUNT(*) FROM " + table;
    	if (selection!=null && selection.length()>0) 	sql += " WHERE "+selection;
    	
    	Cursor c = mDb.rawQuery(sql, selectionArgs);
    	int count = (c.moveToFirst()) ? c.getInt(0) : 0;
    	c.close();
    	
    	return count;
    }
    
    public long createNoteForRestore(
    		long id, String title, String body, long created, int groupid, String trash, 
    		long remindertime, int remindertype) {
		ContentValues values = new ContentValues();
		values.put(FLD_ENTR_ID, id);
		values.put(FLD_ENTR_TITLE, title);
		values.put(FLD_ENTR_BODY, body);
		values.put(FLD_ENTR_CREATED, created);
		values.put(FLD_ENTR_GROUPID, groupid);
		values.put(FLD_ENTR_TRASH, trash);
		values.put(FLD_ENTR_REMINDER_TIME, remindertime);
		values.put(FLD_ENTR_REMINDER_TYPE, remindertype);
		return mDb.insert(TBL_ENTR, null, values);
    }
    
    public long createItemForRestore(long id, long noteid, String title, String fileuri, int type, int color) {
		ContentValues values = new ContentValues();
		values.put(FLD_ITEM_ID, id);
   		values.put(FLD_ITEM_NOTEID, noteid);
   		values.put(FLD_ITEM_TITLE, title);
   		values.put(FLD_ITEM_FILEURI, fileuri);
   		values.put(FLD_ITEM_TYPE, type);
   		values.put(FLD_ITEM_INT0, color);
   		return mDb.insert(TBL_ITEM, null, values);
    }

    public long createLabelMForRestore(String label, int markerid) {
		ContentValues values = new ContentValues();
		values.put(FLD_LBLM_LABEL, label);
		values.put(FLD_LBLM_MARKERID, markerid);
		return mDb.insert(TBL_LBLM, null, values);
    }
    
    public long createLabelIForRestore(long noteid, String label) {
		ContentValues values = new ContentValues();
		values.put(KEY_LBLI_ENTR_ID, noteid);
		values.put(FLD_LBLI_LBLM_ID, label);
		return mDb.insert(TBL_LBLI, null, values);
    }
    
	public long createTaskForRestore(
			long id, String title, 	int completed, int seq, long listId, long due, int recType) {
		ContentValues values = new ContentValues();
		values.put(FLD_TASK_ID, id);
		values.put(FLD_TASK_TITLE, title);
		values.put(FLD_TASK_REMINDER_TYPE, 0);
        values.put(FLD_TASK_COMPLETED, completed);
        values.put(FLD_TASK_SEQ, seq);
        values.put(FLD_TASK_LISTID, listId);
        values.put(FLD_TASK_DUE, due);
        values.put(FLD_TASK_REMINDER_TIME, 0L);
        values.put(FLD_TASK_RECTYPE, recType);
		return mDb.insert(
				TBL_TASK, 
				null, 
				values);
    }
    

    
}
