package com.acj0.orangediaryproa.data;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.preference.PreferenceManager;

import com.acj0.share.SharePref;
import com.acj0.share.utils.FileUtils;
import com.acj0.share.utils.LogUtils;

public class CopyOfPreprocessing {

	private static final String TAG = "Preprocessing";
	
	private Context mContext;
	private DBAdapter mDbHelper;
	private LogUtils mLog;
	
	private SharedPreferences mSharedPref;
	private SharedPreferences.Editor mSharedPrefEdit;
	
	public CopyOfPreprocessing(Context context) {
		
		mContext = context;
		mDbHelper = new DBAdapter(context);
		mSharedPref = PreferenceManager.getDefaultSharedPreferences(context);
		mSharedPrefEdit = mSharedPref.edit();
		mLog = new LogUtils(MyApp.BASE_DIR, LogUtils.LOG_STARTUP);
		
		mLog.logStatus("");
		mLog.logStatus(". . . . . . . . . . . . . . . . . . . . ");
		mLog.logStatus("Start environment check");
		mLog.logStatus("");
	
	}

	public void preprocessing() {
		
		preprocessing1100();
		preprocessing1200();
		
		mDbHelper.open();
		preprocessing2100();

		preprocessing1();
		preprocessing2();
		preprocessing3();
		preprocessing4();
		preprocessing5();
		mDbHelper.close();

		// runnable
		new Thread() {
			@Override
			public void run() {
				
				preprocessing8();
				preprocessing7();
				
			}
		}.start();
		
	}
	
	
	public void preprocessing1100() {
		
		mLog.logStatus("Check 1100: Init pref values");
		
		mSharedPrefEdit.putBoolean(SharePref.KEY_ADMINTOOLS_UNLOCK, false);
		mSharedPrefEdit.commit();
		mLog.logStatus("");
	}



	public void preprocessing1200() {
		mLog.logStatus("Check 1200: Check folders");
		
		int insertCount = 0; 
		
		// Folders
		String[] dir = {
			MyApp.BASE_DIR,
			MyApp.VOICE_DIR,
			MyApp.PHOTO_DIR,
			MyApp.THUMB_DIR,
			MyApp.EXPORT_DIR,
			MyApp.BACKUP_DIR,
			MyApp.AUTOEXP_DIR,
		};
		
		int count=dir.length;
		for (int i=0; i<count; i++) {
			File f = new File(dir[i]);
			if (!f.isDirectory()) {
				insertCount += f.mkdirs() ? 1 : 0;
			}
		}
		
		// nomedia
		String[] nomedia = {
				MyApp.THUMB_DIR,
				MyApp.PHOTO_DIR,
				MyApp.VOICE_DIR,
				MyApp.AUTOEXP_DIR,
			};
			
		count=nomedia.length;
		for (int i=0; i<count; i++) {
			File f = new File(nomedia[i],".nomedia");
			if (!f.isFile()) {
				try { insertCount += f.createNewFile() ? 1 : 0;	} catch (Exception e) {}
			}
		}
		
		LogUtils log = new LogUtils(MyApp.BASE_DIR);
		log.cleanupFileAll();
		
		mLog.logStatus(" . insertCount: "+insertCount);
		mLog.logStatus("");
	}

	public void preprocessing2100() {
		
		mLog.logStatus("Check 2100: Table record count");
		
		String[] tables = DBAdapter.TABLES_IN_USE;
		int count4 = tables.length;
		for (int i=0; i<count4; i++) {
			int count = mDbHelper.countRecord(tables[i], null);
			mLog.logStatus(" . "+tables[i]+": "+count);
		}
		
		Cursor c;
		c = mDbHelper.fetchNotes(null);		mLog.logStatus(" . entr: "+c.getCount());		c.close();
		c = mDbHelper.fetchItems();			mLog.logStatus(" . item: "+c.getCount());		c.close();
		c = mDbHelper.fetchLabelMAll();		mLog.logStatus(" . lblm: "+c.getCount());		c.close();
		c = mDbHelper.fetchLabelIAll();		mLog.logStatus(" . lbli: "+c.getCount());		c.close();
		c = mDbHelper.fetchTodoAll(null);	mLog.logStatus(" . task: "+c.getCount());		c.close();
		mLog.logStatus("");
	}

	public void preprocessing1() {
		
		mLog.logStatus("Check 1: Patch label without parents");
		
		int deleteCount = 0;
		
		Cursor c = mDbHelper.fetchLabelWithoutParent();
		int count = c.getCount();
		for (int i=0; i<count; i++) {
			c.moveToPosition(i);
			deleteCount += mDbHelper.deleteLabelIByEntryId(c.getLong(0));
		}
		c.close();
		
		mLog.logStatus(" . count: "+count);
		mLog.logStatus(" . deleteCount: "+deleteCount);
		mLog.logStatus("");
	}
	
	public void preprocessing2() {
		
		mLog.logStatus("Check 2; fetch item without parents");
		
		int deleteCount = 0;
		
		Cursor c = mDbHelper.fetchItemWithoutParent();
		int count = c.getCount();
		for (int i=0; i<count; i++) {
			c.moveToPosition(i);
			deleteCount += mDbHelper.deleteItemByNoteid(c.getLong(0));
		}
		c.close();
		
		mLog.logStatus(" . count: "+count);
		mLog.logStatus(" . deleteCount: "+deleteCount);
		mLog.logStatus("");
	}

	public void preprocessing3() {
		
		mLog.logStatus("Check 3: null trash update count");
		
		int updateCount = mDbHelper.updateNullTrash();
		mLog.logStatus(" . updateCount: "+updateCount);
		mLog.logStatus("");
	}
	
	public void preprocessing4() {
		
		mLog.logStatus("Check 4: comma in tag field");
		
		List<String> list = new ArrayList<String>();
		
		Cursor c = mDbHelper.fetchLabelMSpecial();
		int count = c.getCount();
		for (int i=0; i<count; i++) {
			c.moveToPosition(i);
			String label = c.getString(0);
			list.add(label);
		}
		c.close();

		int updateCount1 = 0;
		int updateCount2 = 0;
		
		count = list.size();
		for (int i=0; i<count; i++) {
			String oldS = list.get(i);
			String newS = list.get(i).replaceAll(",", "");
			updateCount1 = mDbHelper.updateLabelM(oldS, newS);
			updateCount2 = mDbHelper.updateLabelI(oldS, newS);
		}
		
		mLog.logStatus(" . count: "+count);
		mLog.logStatus(" . updateCount lblM: "+updateCount1);
		mLog.logStatus(" . updateCount lblI: "+updateCount2);
		mLog.logStatus("");
	}
	
	public void preprocessing5() {
		
		mLog.logStatus("Check 5: lost parent");
		
		int updateCount1 = 0;

		List<Long> listId = new ArrayList<Long>();
		List<Long> listParents = new ArrayList<Long>();
		
		Cursor c = mDbHelper.fetchLabel2MAllByType(LabelOper2.TYPE_FLD);
		int count = c.getCount();
		for (int i=0; i<count; i++) {
			c.moveToPosition(i);
			listId.add(c.getLong(0));
			listParents.add(c.getLong(4));
		}
		c.close();
		
		count = listParents.size();
		for (int i=0; i<count; i++) {
			long parent = listParents.get(i);
			if (listId.contains(parent)) {
				// do nothing
			}
			else {
				mLog.logStatus(" . updating lbl2M: "+parent+" to Root");
				updateCount1 += mDbHelper.updateLabel2MParentByParent(parent, 0);
			}
		}
		
		mLog.logStatus(" . updateCount1: "+updateCount1);
		mLog.logStatus("");
	}
	
	public void preprocessing7() {
		
		mLog.logStatus("Check 7: Fix null value in folder id field");
		
		int updateCount1 = 0;
		
		DBAdapter dbHelper = new DBAdapter(mContext);
		dbHelper.open();
		
		dbHelper.updateSpecialEntryFolder001();
		
		dbHelper.close();
		mLog.logStatus(" . count1, updated: "+updateCount1);
		mLog.logStatus("");
		
	}
	
	// bring images/voice to local
	public void preprocessing8() {
		
		mLog.logStatus("Check 8: Bring attachment to local");
		
		int updateCount1 = 0;
		int updateCount2 = 0;
		int updateCount3 = 0;
		
		DBAdapter dbHelper = new DBAdapter(mContext);
		dbHelper.open();
		
		// 1. check default storage
		String[] columns = {DBAdapter.FLD_ITEM_ID, DBAdapter.FLD_ITEM_FILEURI};
		String type = "0,1,2,3,4,5";
		String[] selectionArgs = {	MyApp.ROOT_DIR+"/%"	};
		Cursor c = dbHelper.fetchItemSpecial001(columns, type, selectionArgs);
		int selectCount1 = c.getCount();
		for (int i=0; i<selectCount1; i++) {
			c.moveToPosition(i);
	    	long itemId = c.getLong(0);
	    	String fileuri = c.getString(1);
	    	
	    	if (fileuri!=null && fileuri.length()>0) {
	    		mLog.logStatus(" . fixing1: from. "+fileuri);
	    		String newFileUri = FileUtils.fixRoot(fileuri);		// return uri with correct root
	    		if (newFileUri!=null) {
	    			File f = new File(newFileUri);
				    if (f.isFile()) {
				    	updateCount1 += dbHelper.updateItemFileuri(itemId, newFileUri);
			    		mLog.logStatus(" . fixing1: to. "+newFileUri);
				    }
	    		}
	    	}
		}
		c.close();
		
		// 2. check images
		type = "0,1";
		selectionArgs[0] = "%"+Multimedia.SEED_IMAGE+"%";
		c = dbHelper.fetchItemSpecial001(columns, type, selectionArgs);
		int selectCount2 = c.getCount();
		for (int i=0; i<selectCount2; i++) {
			c.moveToPosition(i);
	    	long itemId = c.getLong(0);
	    	String fileuri = c.getString(1);
	    	
	    	mLog.logStatus(" . fixing2: from. "+fileuri);
	    	String local = Multimedia.makeLocal(fileuri, ItemOper.TYPE_IMAGE);
	    	if (local!=null) {
	    		updateCount2 += dbHelper.updateItemFileuri(itemId, local);
				mLog.logStatus(" . fixing2: to. "+local);
	    	}
	    	
		}
		c.close();
		
		// 3. check audio
		type = "3,4";
		selectionArgs[0] = "%"+Multimedia.SEED_AUDIO+"%";
		c = dbHelper.fetchItemSpecial001(columns, type, selectionArgs);
		int selectCount3 = c.getCount();
		for (int i=0; i<selectCount3; i++) {
			c.moveToPosition(i);
	    	long itemId = c.getLong(0);
	    	String fileuri = c.getString(1);
	    	
	    	mLog.logStatus(" . fixing3: from. "+fileuri);
	    	String local = Multimedia.makeLocal(fileuri, ItemOper.TYPE_AUDIO);
	    	if (local!=null) {
	    		updateCount3 += dbHelper.updateItemFileuri(itemId, local);
				mLog.logStatus(" . fixing3: to. "+local);
	    	}
	    	
		}
		c.close();
		
		dbHelper.close();
		mLog.logStatus(" . count1, selected:"+selectCount1+", updated: "+updateCount1);
		mLog.logStatus(" . count2, selected:"+selectCount2+", updated: "+updateCount2);
		mLog.logStatus(" . count3, selected:"+selectCount3+", updated: "+updateCount3);
		mLog.logStatus("");
		
	}
	
	public String getPathWithoutRoot(String path) {
		
		if (path==null || path.length()==0)		return null;
		
		String[] x = path.split("/");
		if (x==null || x.length<2)	return null;

		int startfrom = 1;
		if (path.contains("/mnt/sdcard"))				startfrom = 3;
		else if (path.contains("/storage/sdcard"))		startfrom = 3;
		else if (path.contains("/storage/emulated/"))	startfrom = 4;
		
		if (x==null || x.length<startfrom+1)	return null;
		
		StringBuilder sb = new StringBuilder();
		for (int i=startfrom; i<x.length; i++) {
			sb.append("/"+x[i]);
		}
		
		mLog.logStatus(" . fixing1: info. "+startfrom+", "+x[0]+"."+x[1]+"."+x[2]);
		return sb.toString();
		
	}
	
	
}
