package com.acj0.orangediaryproa.mod.googleapi;

import java.io.File;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

import com.acj0.orangediaryproa.ListEntry;
import com.acj0.orangediaryproa.MyPref;
import com.acj0.orangediaryproa.R;
import com.acj0.orangediaryproa.data.DBAdapter;
import com.acj0.orangediaryproa.data.EntryData;
import com.acj0.orangediaryproa.data.EntryOper;
import com.acj0.orangediaryproa.data.Export1;
import com.acj0.orangediaryproa.data.Intent1;
import com.acj0.orangediaryproa.data.ItemOper;
import com.acj0.orangediaryproa.data.LabelOper;
import com.acj0.orangediaryproa.data.Multimedia;
import com.acj0.orangediaryproa.data.MyApp;
import com.acj0.orangediaryproa.mod.alarm.Reminder;
import com.acj0.orangediaryproa.mod.import1.CsvFile;
import com.acj0.orangediaryproa.mod.import1.MyApiDocList;
import com.acj0.orangediaryproa.mod.import1.MyApiPicasa;
import com.acj0.share.ShareApp;
import com.acj0.share.ShareIntent;
import com.acj0.share.mod.api.google.ApiAuthV2;
import com.acj0.share.mod.googleapi.ApiBlogger;
import com.acj0.share.mod.googleapi.ApiPicasa;
import com.acj0.share.utils.IOUtils;
import com.acj0.share.utils.LogUtils;
import com.acj0.share.utils.ViewLogFile;

public class UploaderGDocsService extends Service {

	// mostly same... check PACKAGE DEPENDENT part at the bottom 2014-04-22
	private static final String TAG = "UploaderGDocsService";

	// MV
	private int mExtraFormat;
	private int mExtraListType;
	private String mExtraLabelId;
	private String mExtraSrchSelSql;
	private String[] mExtraSrchSelArg;

	private ApiAuthV2 mApiAuth;
	private MyApiDocList mApiDocList;
	private MyApiPicasa mApiPicasa;
	private LogUtils mLog;

	public int mErrorCode;
	public String mErrorMssg;

	private DBAdapter mDbHelper;
	private EntryOper mEntry1;
	private Cursor mCursor;
	private StringBuilder mSB;

	// Preferences
	private int mPrefEntrySortOrder;

	@Override
	public void onCreate() {
		if (MyApp.DEBUG) Log.e(TAG, "onCreate");
	}

	@Override
	public void onStart(Intent intent, int startId) {
		super.onStart(intent, startId);
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {

		if (MyApp.DEBUG) Log.e(TAG, "flag: " + flags);
		if (MyApp.DEBUG) Log.e(TAG, "startId: " + startId);

		if (intent==null) {
			mLog.logStatus("Starting failed with null");
		}
		
		// Extras
		Bundle extras = intent.getExtras();
		if (extras != null) {
			mExtraFormat = extras.getInt(ShareIntent.EXTRA_EXPORT_FORMAT);
			mExtraListType = extras.getInt(Intent1.EXTRA_LIST_TYPE);
			mExtraLabelId = extras.getString(Intent1.EXTRA_LABEL_ID);
			mExtraSrchSelSql = extras.getString(Intent1.EXTRA_SRCH_SEL_SQL);
			mExtraSrchSelArg = extras.getStringArray(Intent1.EXTRA_SRCH_SEL_ARG);
		}

		Thread thread = new Thread(null, mTask, "BackupService");
		thread.start();

		if (ShareApp.DEBUG) Log.e(TAG, "onStart-end");
		return super.onStartCommand(intent, flags, startId);
	}

	@Override
	public void onDestroy() {
		if (MyApp.DEBUG) Log.e(TAG, "onDestroy");
	}

	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

	private Runnable mTask = new Runnable() {
		public synchronized void run() {

			// MV
			Context context = getBaseContext();

			mLog = new LogUtils(MyApp.BASE_DIR, LogUtils.LOG_GDOCS);
			mDbHelper = new DBAdapter(context);
			
			mLog.logStatus("");
			mLog.logStatus(". . . . . . . . . . . . . . . . . . . . ");
			mLog.logStatus("Starting upload job");
			mLog.logStatus("");
			
			mDbHelper.open();

			showNotification(2);
			
			mApiAuth = new ApiAuthV2(context);
			mApiAuth.getAccessTokenByRefresh();
			if (mApiAuth.mErrorCode > 0) {
				mErrorCode = mApiAuth.mErrorCode;
				mErrorMssg = mApiAuth.mErrorMssg;
				showNotification(9);
				return;
			}
			
			mApiDocList = new MyApiDocList(mApiAuth.mAccessToken);
			
			if (mExtraFormat == Export1.FORMAT_HTM) {
				mApiPicasa = new MyApiPicasa(mApiAuth.mAccessToken);
				mApiPicasa.loadPhotoList();
			}
			
			mEntry1 = new EntryOper(context, mDbHelper);
			
			SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
			mPrefEntrySortOrder = sp.getInt(MyPref.KEY_LISTENTRY_SORT_ORDER, MyPref.VAL_LISTENTRY_SORT_ORDER);

			getCursor();
			if (mCursor.getCount() == 0) {
				mErrorCode = 99;
				mErrorMssg = "No items to upload";
				showNotification(9);
				return;
			}

			mLog.logStatus(" .identified " + mCursor.getCount() + " items to upload.");

			// preparing document body
			mSB = new StringBuilder();
			uploadImages();
			if (mErrorCode > 0) {
				showNotification(9);
				return;
			}

			// start upload
			showNotification(1);

			String exportTempFile =
					Export1.EXPORT_FILE_PREFIX[Export1.SOURCE_ENT] +
							Export1.SDF1.format(System.currentTimeMillis()) + "." +
							Export1.FILE_EXTENSION[mExtraFormat];

			File file = new File(MyApp.EXPORT_DIR, exportTempFile);
			IOUtils.stringIntoFile(file, mSB.toString(), false, false);
			String folder = MyApp.DEFAULT_DIR;
			String contentType = null;
			switch (mExtraFormat) {
			case Export1.FORMAT_HTM:
				contentType = MyApiDocList.CONTENT_TYPE_HTM;
				break;
			case Export1.FORMAT_CSV:
				contentType = MyApiDocList.CONTENT_TYPE_CSV;
				break;
			}
			mApiDocList.uploadDocumentAuto(file, folder, contentType);

			mErrorCode = mApiDocList.mErrorCode;
			mErrorMssg = mApiDocList.mErrorMssg;
			showNotification(9);
			return;

		}
	};

	private int uploadCount = 0;

	public void uploadImages() {

		uploadCount = 0;
		mSB = new StringBuilder();
		if (mExtraFormat == Export1.FORMAT_HTM) {
			showNotification(0);
		}
		else {
			mSB.append(CsvFile.getEntryHeaderString());
		}

		int count = mCursor.getCount();
		for (int i = 0; i < count; i++) {
			mCursor.moveToPosition(i);
			long noteid = mCursor.getLong(0);
			
			// add text
			EntryData ed = mEntry1.getEntryDataForExport(mCursor);
			
			// attachment Url for images
			StringBuilder sbImage = new StringBuilder();
			if (mExtraFormat == Export1.FORMAT_HTM) {
				
				// get images uploaded. first 
				String[] columns = { DBAdapter.FLD_ITEM_FILEURI };
				String types = "0,1";
				Cursor c = mDbHelper.fetchItemByNoteidType(columns, noteid, types);
				int countj = c.getCount();
				for (int j = 0; j < countj; j++) {
					c.moveToPosition(j);
					uploadCount++;
					String fileuri = c.getString(0);
					fileuri = Multimedia.resolveAbsolutePath(ItemOper.TYPE_IMAGE, fileuri);
					File f = new File(fileuri);
					if (f.isFile() && 
							(	f.getName().toLowerCase().contains(".jpg") || 
								f.getName().toLowerCase().contains(".png") || 
								f.getName().toLowerCase().contains(".bmp") || 
								f.getName().toLowerCase().contains(".jpeg")		)) {
						mLog.logStatus(" .processing image: "+fileuri.replaceAll(MyApp.PHOTO_DIR, ""));
						String imageHtml = processPicasa(fileuri);			// Picasa upload
						mLog.logStatus(" ."+mErrorMssg);
						if (mErrorCode > 0) {
							return;
						}
						sbImage.append(imageHtml);
					}
				}
				c.close();
				
				if (sbImage.length()>0) {
					ed.picasaUrl = sbImage.toString();
				}
			}
			
			mSB.append(ed.getExportString(mExtraFormat));

		}
	}

	public String processPicasa(String imageFile) {

		File file = new File(imageFile);
		if (file == null || !file.isFile()) {
			mErrorCode = 0;
			mErrorMssg = "error: File not found";
			return ApiBlogger.IMAGE_TEMPLATE
				.replaceAll("xxxxx", ApiPicasa.IMAGE_NOT_AVAIL_URL)
				.replaceAll("yyyyy", ApiPicasa.IMAGE_NOT_AVAIL_URL) + "<br><br><br>";
		}
		
		String contentUrl = mApiPicasa.postPhotoMetaSmart(file);

		if (mApiPicasa.mErrorCode > 0) {
			mErrorCode = mApiPicasa.mErrorCode;
			mErrorMssg = "error: "+mApiPicasa.mErrorMssg;
			return ApiBlogger.IMAGE_TEMPLATE
				.replaceAll("xxxxx", ApiPicasa.IMAGE_NOT_AVAIL_URL)
				.replaceAll("yyyyy", ApiPicasa.IMAGE_NOT_AVAIL_URL) + "<br><br><br>";
		}

		if (contentUrl == null || contentUrl.length()==0) {
			mErrorCode = 998;
			mErrorMssg = "error: Image url not received";
			return ApiBlogger.IMAGE_TEMPLATE
				.replaceAll("xxxxx", ApiPicasa.IMAGE_NOT_AVAIL_URL)
				.replaceAll("yyyyy", ApiPicasa.IMAGE_NOT_AVAIL_URL) + "<br><br><br>";
		}
		
		mErrorCode = 0;
		mErrorMssg = "Success: "+contentUrl;

		return ApiBlogger.IMAGE_TEMPLATE
				.replaceAll("xxxxx", contentUrl)
				.replaceAll("yyyyy", contentUrl) + "<br><br><br>";

	}
	
	private void showNotification(int progress) {

		String contentTitle = "Google Drive uploader";
		String contentText = "";

		if (progress == 0) {
			contentText = "Starting image upload";
			mLog.logStatus("");
			mLog.logStatus(contentText);
		}
		else if (progress == 1) {
			contentText = "Starting text data upload";
			mLog.logStatus("");
			mLog.logStatus(contentText);
		}
		else if (progress == 2) {
			contentText = "Preparing upload";
			mLog.logStatus(contentText);
		}
		else if (progress == 9) {
			if (mErrorCode == 0) {
				contentText = " .upload completed.";
			}
			else {
				contentText = " .error. " + mErrorCode + ". " + mErrorMssg;
			}
			mLog.logStatus(contentText);
		}

		String tickerText = MyApp.APP_NAME_LONG+". " + contentText;

		Intent intent = new Intent(this, ViewLogFile.class);
		intent.putExtra(ShareIntent.EXTRA_LOG_FILE, mLog.getLogFilFullpath());
		intent.putExtra(ShareIntent.EXTRA_LOG_TITLE, "GDrive upload log");
		int id = R.string.service_backup;
		int icon = R.drawable.notif_letter_or;

		Reminder rem = new Reminder(this);
		rem.showNotification(id, id, intent, icon, tickerText, contentTitle, contentText);

		if (progress == 9) {
			if (mCursor != null) mCursor.close();
			if (mDbHelper!=null) mDbHelper.close();
			stopSelf();
			mLog.logStatus("");
			mLog.logStatus("End of upload job");
			mLog.logStatus(". . . . . . . . . . . . . . . . . . . . ");
			mLog.logStatus("");
		}
		else {
			// keep going
		}

	}
	
	// ========================================
	// PACKAGE DEPENDENT
	// ========================================

	public void getCursor() {

		if (mCursor!=null)	mCursor.close();
		
		String[] columns = DBAdapter.PROJ_ENTR;
		String orderBy = ListEntry.SORT_ORDER_KEY[mPrefEntrySortOrder];
		
		switch (mExtraListType) {
		case ListEntry.MODE_TAG:
			if (mExtraLabelId.equals(LabelOper.NO_TAGS)) 
				mCursor = mDbHelper.fetchNoteWithoutLabel(columns,orderBy);
			else 
				mCursor = mDbHelper.fetchNoteByTag(columns, mExtraLabelId,orderBy);
			break;
		case ListEntry.MODE_SRCH:
			mCursor = mDbHelper.fetchNoteByQuery(columns, mExtraSrchSelSql, mExtraSrchSelArg, orderBy);
			break;
		case ListEntry.MODE_ALL:
			mCursor = mDbHelper.fetchAllNote1(columns, orderBy);
			break;
		}

	}
	

}